#include "HotKeyUI.h"
#include <CommCtrl.h>

CHotKeyUI* pHotKeyUICurrentUI = NULL;
CHotKeyUI::CHotKeyUI(void)
{
}

CHotKeyUI::~CHotKeyUI(void)
{
}

//-----------------------------------------------------------------------------
// Name: Show
// Object: show the filter dialog box
// Parameters :
//     in  : HINSTANCE hInstance : application instance
//           HWND hWndDialog : main window dialog handle
//     out :
//     return : 
//-----------------------------------------------------------------------------
INT_PTR CHotKeyUI::Show(HINSTANCE hInstance,HWND hWndDialog,COptions* pOptions)
{
    if (pHotKeyUICurrentUI)
    {
        ::SetForegroundWindow(pHotKeyUICurrentUI->hWndDialog);
        ::FlashWindow(pHotKeyUICurrentUI->hWndDialog,TRUE);
        return 0;
    }

    
    INT_PTR Ret;
    CHotKeyUI* pHotKeyUI=new CHotKeyUI();
    pHotKeyUICurrentUI = pHotKeyUI;

    pHotKeyUI->hInstance=hInstance;
    pHotKeyUI->hParentHandle=hWndDialog;
    pHotKeyUI->pOptions=pOptions;

    Ret=::DialogBoxParam(pHotKeyUI->hInstance,(LPCTSTR)IDD_DIALOG_HOTKEY,NULL,(DLGPROC)CHotKeyUI::WndProc,(LPARAM)pHotKeyUI);

    pHotKeyUICurrentUI = NULL;
    delete pHotKeyUI;

    return Ret;
}

void CHotKeyUI::OnInit()
{
    // set existing hotkey
    HWND hHotKeyCtl;
    hHotKeyCtl = ::GetDlgItem(this->hWndDialog,IDC_HOTKEY_READ_CLIPBOARD);
    ::SendMessage(hHotKeyCtl, (UINT) HKM_SETHOTKEY, this->pOptions->ReadClipboardHotKey ,0);
    hHotKeyCtl = ::GetDlgItem(this->hWndDialog,IDC_HOTKEY_START_STOP);
    ::SendMessage(hHotKeyCtl, (UINT) HKM_SETHOTKEY, this->pOptions->StartStopHotKey ,0);
    hHotKeyCtl = ::GetDlgItem(this->hWndDialog,IDC_HOTKEY_MOUSE_TO_WINDOW_MODE);
    ::SendMessage(hHotKeyCtl, (UINT) HKM_SETHOTKEY, this->pOptions->SwitchMouseWindowModeHotKey ,0);

    if (this->pOptions->bUseHotKeys)
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_CHECK_USE_HOTKEYS),TRUE);
    CDialogHelper::EnableGroup(::GetDlgItem(this->hWndDialog,IDC_GROUP_HOTKEYS),this->pOptions->bUseHotKeys);
}
void CHotKeyUI::OnClose()
{
    this->OnCancel();
}
void CHotKeyUI::OnCancel()
{
    ::EndDialog(this->hWndDialog,0);
}
void CHotKeyUI::OnOk()
{
    // save existing hotkey
    HWND hHotKeyCtl;
    hHotKeyCtl = ::GetDlgItem(this->hWndDialog,IDC_HOTKEY_READ_CLIPBOARD);
    // HKM_GETHOTKEY MSDN :
    //Returns the virtual key code and modifier flags. The LOBYTE of the LOWORD is the virtual key code of the hot key. The HIBYTE of the LOWORD is the key modifier that specifies the keys that define a hot key combination. The modifier flags can be a combination of the following values.
    //    Value	Meaning
    //    HOTKEYF_ALT	ALT key
    //    HOTKEYF_CONTROL	CONTROL key
    //    HOTKEYF_EXT	Extended key
    //    HOTKEYF_SHIFT	SHIFT key
    this->pOptions->ReadClipboardHotKey = ::SendMessage(hHotKeyCtl, (UINT) HKM_GETHOTKEY, 0 ,0);
    hHotKeyCtl = ::GetDlgItem(this->hWndDialog,IDC_HOTKEY_START_STOP);
    this->pOptions->StartStopHotKey = ::SendMessage(hHotKeyCtl, (UINT) HKM_GETHOTKEY, 0 ,0);
    hHotKeyCtl = ::GetDlgItem(this->hWndDialog,IDC_HOTKEY_MOUSE_TO_WINDOW_MODE);
    this->pOptions->SwitchMouseWindowModeHotKey = ::SendMessage(hHotKeyCtl, (UINT) HKM_GETHOTKEY, 0 ,0);
    
    this->pOptions->bUseHotKeys = CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_USE_HOTKEYS));

    ::EndDialog(this->hWndDialog,1);
}

//-----------------------------------------------------------------------------
// Name: GetAssociatedObject
// Object: Get object associated to window handle
// Parameters :
//     in  : HWND hWndDialog : handle of the window
//     out :
//     return : associated object if found, NULL if not found
//-----------------------------------------------------------------------------
CHotKeyUI* CHotKeyUI::GetAssociatedObject(HWND hWndDialog)
{
    return (CHotKeyUI*)GetWindowLongPtr(hWndDialog,GWLP_USERDATA);
}
//-----------------------------------------------------------------------------
// Name: WndProc
// Object: dialog callback of the dump dialog
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK CHotKeyUI::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch (uMsg)
    {
    case WM_INITDIALOG:
        {
            CHotKeyUI* pHotKeyUI=(CHotKeyUI*)lParam;
            pHotKeyUI->hWndDialog=hWnd;

            ::SetWindowLongPtr(hWnd,GWLP_USERDATA,(LONG_PTR)pHotKeyUI);

            pHotKeyUI->OnInit();
            // load dlg icons
            CDialogHelper::SetIcon(hWnd,IDI_ICON_HOTKEYS);
        }
        break;
    case WM_CLOSE:
        {
            CHotKeyUI* pHotKeyUI=CHotKeyUI::GetAssociatedObject(hWnd);
            if (!pHotKeyUI)
                break;
            pHotKeyUI->OnClose();
        }
        break;

    case WM_COMMAND:
        {
            CHotKeyUI* pHotKeyUI=CHotKeyUI::GetAssociatedObject(hWnd);
            if (!pHotKeyUI)
                break;

            switch (LOWORD(wParam))
            {
            case IDOK:
                pHotKeyUI->OnOk();
                break;
            case IDCANCEL:
                pHotKeyUI->OnCancel();
                break;
            case IDC_CHECK_USE_HOTKEYS:
                {
                    BOOL bEnableGroup = CDialogHelper::IsButtonChecked(::GetDlgItem(pHotKeyUI->hWndDialog,IDC_CHECK_USE_HOTKEYS));
                    CDialogHelper::EnableGroup(::GetDlgItem(pHotKeyUI->hWndDialog,IDC_GROUP_HOTKEYS),bEnableGroup);
                }
                break;
            }
        }
        break;

    default:
        return FALSE;
    }
    return TRUE;
}
