#include "ObjectNameFromMouse.h"

#include <OleAcc.h>
#pragma comment(lib,"Oleacc.lib")

//*pObjectName : if not null, must be free using "free"
//*pObjectTextRole : if not null, must be free using "free"
//*pObjectTextValue : if not null, must be free using "free"
BOOL CObjectNameFromMouse::GetObjectNameFromMouse(WCHAR** pObjectName, WCHAR** pObjectTextRole, WCHAR** pObjectTextValue)
{
    return CObjectNameFromMouse::GetObjectNameFromMouseOrWindow(pObjectName, pObjectTextRole, pObjectTextValue,NULL,FALSE);
}
BOOL CObjectNameFromMouse::GetObjectNameFromWindow(HWND hWindow,WCHAR** pObjectName, WCHAR** pObjectTextRole, WCHAR** pObjectTextValue)
{
    return CObjectNameFromMouse::GetObjectNameFromMouseOrWindow(pObjectName, pObjectTextRole, pObjectTextValue,hWindow,TRUE);
}

BOOL CObjectNameFromMouse::GetObjectNameFromMouseOrWindow(WCHAR** pObjectName, WCHAR** pObjectTextRole, WCHAR** pObjectTextValue,HWND hWindow,BOOL bFromWindowHandle)
{
    *pObjectName = 0;
    *pObjectTextRole = 0;
    *pObjectTextValue = 0;

    BOOL bSuccess = FALSE;

    // Setup variables for interface pointers and return value.
    IAccessible* pAccessible;
    HRESULT      hResult;
    VARIANT      varChild;
    VARIANT		 varRole;
    BSTR		 bstrName;

    pAccessible = NULL;
    varChild.vt = VT_EMPTY;
    varRole.vt = VT_EMPTY;
    bstrName = NULL;

    if (bFromWindowHandle)
    {
        hResult = ::AccessibleObjectFromWindow(hWindow,/*OBJID_WINDOW*/ OBJID_CLIENT, IID_IAccessible,(void**)&pAccessible);
        varChild.vt = VT_I4;
        varChild.lVal = CHILDID_SELF;
    }
    else
    {
        // Get the current cursor location.
        POINT ptCursor;
        ::GetCursorPos(&ptCursor);

        // See if there is an accessible object under the cursor.
        hResult = ::AccessibleObjectFromPoint(ptCursor, &pAccessible, &varChild);
    }

    if ( (hResult != S_OK) || (pAccessible == 0) )
        goto CleanUp;

    // Query on the object's name.
    hResult = pAccessible->get_accName(varChild, &bstrName);
    if ( (hResult == S_OK) && (bstrName != 0) )
    {
        if (*bstrName != 0)
            *pObjectName = wcsdup(bstrName);
        ::SysFreeString(bstrName);
    }

    // get object value
    hResult = pAccessible->get_accValue(varChild,&bstrName);
    if ( (hResult == S_OK) && (bstrName != 0) )
    {
        if (*bstrName != 0)
            *pObjectTextValue = wcsdup(bstrName);
        ::SysFreeString(bstrName);
    }

    // if no name and no value
    if ( (*pObjectName==0) && (*pObjectTextValue==0) )
        goto CleanUp;

    // Query role name
    hResult = pAccessible->get_accRole (varChild,&varRole);
    if (hResult == S_OK && varRole.vt == VT_I4)
    {
        UINT   iRoleLength;

        // Find out how long the role text string is.
        iRoleLength = ::GetRoleTextW(varRole.lVal, NULL, 0);

        // Allocate memory for the string. Add one byte to
        // the length you got in the previous call to make room
        // for the null character.
        *pObjectTextRole = (WCHAR*) malloc( (iRoleLength+1) * sizeof(WCHAR) );

        // Get the string.
        ::GetRoleTextW(varRole.lVal, *pObjectTextRole, iRoleLength +1);
    }

    bSuccess = TRUE;
CleanUp:
    if (pAccessible)
        pAccessible->Release();
    if (!bFromWindowHandle)
        ::VariantClear(&varChild);
    return bSuccess;
}