#pragma once

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0501 // for xp os
#endif

#include "../Tools/gui/NotifyIcon/NotifyIcon.h"
#include "../Tools/gui/Menu/PopUpMenu.h"
#include "../Tools/SoftwareUpdate/SoftwareUpdate.h"
#include "../Tools/Clipboard/ClipboardViewer.h"

#include <windows.h>
#include "resource.h"
#include "ObjectNameFromMouse.h"
#include "Options.h"
#include "Voice.h"
#include "about.h"
#include "ReadUI.h"
#include "HotKeyUI.h"
#include "VoiceOptions.h"
#include "SelectWindowUI.h"

// defines for update checking
#define ScreenReader_PAD_Url _T("http://jacquelin.potier.free.fr/screenreader/ScreenReader.xml")
#define ScreenReaderVersionSignificantDigits 3
#ifdef _WIN64
#define ScreenReaderDownloadLink _T("http://jacquelin.potier.free.fr/exe/ScreenReader64.zip")
#else
#define ScreenReaderDownloadLink _T("http://jacquelin.potier.free.fr/exe/ScreenReader.zip")
#endif

// other defines
#define CScreenReader_CLASS_NAME                        _T("CScreenReaderClassName")
#define CScreenReader_VOICE_NAME_MAX_LEN                COption_VOICE_NAME_MAX_LEN
#define CScreenReader_READ_CLIPBOARD_HOTKEY_ID          0
#define CScreenReader_START_STOP_HOTKEY_ID              1
#define CScreenReader_MOUSE_TO_WINDOW_MODE_HOTKEY_ID    2

#define CScreenReader_ENDING_SENTENCE_CHARS_W L"?.!;:\r\n"
#define CScreenReader_ENDING_BLOCKS_CHARS_W (CScreenReader_ENDING_SENTENCE_CHARS_W L",(){}\"'\t\f\v")

// Enable Visual Style
#if defined _AMD64
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='amd64' publicKeyToken='6595b64144ccf1df' language='*'\"")
#elif defined _WIN64
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='ia64' publicKeyToken='6595b64144ccf1df' language='*'\"")
#else
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='x86' publicKeyToken='6595b64144ccf1df' language='*'\"")
#endif

class CScreenReader:public CNotifyIconEventsBase
{
protected:
    typedef enum tagReadWay {ReadWay_MouseMode,ReadWay_WindowMode};
    typedef enum tagReadContentStep{
                                    ReadStep_NoContentChangeReadNextSentence,   // Read sentences on by one until end of text
                                    ReadStep_NoContentChangeAllSentencesRead,   // --> All sentences read, nothing to do more

                                    ReadStep_ContentModifiedReadLetter,         // content has changed, if single letter read it
                                    ReadStep_ContentModifiedReadWord,           // after having read the changed letter, read the changed word if no other modification
                                    ReadStep_ContentModifiedReadBlock,          // after having read the changed word, read the changed block if no other modification
                                    ReadStep_ContentModifiedReadSentence,       // after having read the changed block, read the changed sentence if no other modification
                                    ReadStep_ContentModifiedReadNextChangedSentence, // if multiple sentence have been changed, read modified sentences if no other modification
                                    ReadStep_ContentModifiedAllChangedSentencesRead // all changed sentences have been read

                                    };
    HINSTANCE hInstance;
    HWND hWndDialog;
    CVoice* pVoice;
    COptions* pOptions;
    CNotifyIcon* pNotifyIcon;
    HWND hWndLastReadWindowHandle;  // handle of last window treated
    WCHAR* LastWindowContent;       // fully allocated buffer
    SIZE_T LastWindowContentSize;   // LastWindowContent size
    tagReadContentStep ReadStep;    // current read step, allow to know next action to execute
    WCHAR* LastChangeStart;         // if any change !=0, pointer inside LastWindowContent.
    SIZE_T LastChangeSize;          // change size
    WCHAR* LastReadStart;           // pointer inside LastWindowContent (points to last position provided to pVoice->Read)
    SIZE_T LastReadSize;            // size of data provided to the last pVoice->Read operation
    WCHAR* LastTextToRead;          // last text to read put in read queue
    tagReadWay ReadWay;
    HWND hWndSelectedWindow;

    CPopUpMenu* pPopUpMenu;
    CPopUpMenu* pPopUpMenuVoices;
    BOOL bHotKeyRegistered;
    CClipboardViewer ClipBoardViewer;
    BOOL WaitCurrentReadToComplete;
    HANDLE CompleteEvent;

    UINT MenuIdRead;
    UINT MenuIdReadClipboard;
    UINT MenuIdAutoReadClipBoardContent;
    UINT MenuIdEnabled;
    UINT MenuIdReadRole;
    UINT MenuIdVoiceOptions;
    UINT MenuIdAbout;
    UINT MenuIdMakeADonation;
    UINT MenuIdCheckForUpdate;
    UINT MenuIdExit;
    UINT CurrentVoiceMenuId;
    UINT MenuIdHotKeys;
    UINT MenuIdSelectWindow;

    BOOL EndSessionProcessed;
    HMODULE hModuleUser32;
    typedef BOOL (WINAPI *pfShutdownBlockReasonCreate)(HWND hWnd,LPCWSTR pwszReason);
    pfShutdownBlockReasonCreate pShutdownBlockReasonCreate;
    typedef BOOL (WINAPI *pfShutdownBlockReasonDestroy)(HWND hWnd);
    pfShutdownBlockReasonDestroy pShutdownBlockReasonDestroy;

    BOOL NotifyIconCallBack(LPPOINT pCursorPos);
    virtual BOOL OnNotifyIconClick(CNotifyIcon* pSender,LPPOINT pCursorPos){UNREFERENCED_PARAMETER(pSender);return this->NotifyIconCallBack(pCursorPos);}
    virtual BOOL OnNotifyIconRightClick(CNotifyIcon* pSender,LPPOINT pCursorPos){UNREFERENCED_PARAMETER(pSender);return this->NotifyIconCallBack(pCursorPos);}
    virtual BOOL OnNotifyIconDoubleClick(CNotifyIcon* pSender,LPPOINT pCursorPos){UNREFERENCED_PARAMETER(pSender);return this->NotifyIconCallBack(pCursorPos);}

    static VOID CALLBACK TimerProc(HWND hWnd,UINT uMsg,UINT_PTR idEvent,DWORD dwTime);
    static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    void OnClose();
    void OnInit();
    static CScreenReader* GetAssociatedObject(HWND hWndDialog);
    void StartStopReading(BOOL bStart);
    void RegisterHotKeys();
    void UnregisterHotKeys();
    BOOL WaitApplicationEnd();
    BOOL Exit();
    BOOL ReadAccessibleObject(BOOL bFromMouse);
    BOOL GetChangePosition(WCHAR* NewContent,WCHAR** pChangeStart,SIZE_T* pChangeSize);
    BOOL UpdateReadBufferToReadCurrentOrPreviousWord();
    BOOL UpdateReadBufferToReadCurrentOrPreviousSentence();
    BOOL UpdateReadBufferToReadCurrentOrPreviousBlock();
    BOOL UpdateReadBufferToReadEndOfSentence();
    static DWORD WINAPI ReadingThread( LPVOID lpParam );
    static BOOL CallBackEnumVoices(WCHAR* TTSName,PVOID UserParam);
    void HotKeyToVkeyAndModifier(IN DWORD HotKey,OUT BYTE* pVkeyCode,OUT BYTE* pfsModifier);
    void UpdateHotKey();
    void ReadClipBoard();
    void CheckForUpdate();
    void MakeADonation();
    static void OnClipBoardChangeCallBack(PVOID UserParam);
    void OnClipBoardChangeCallBack();
    HWND SmallestWindowFromPoint( const POINT point );
    void SwitchMouseWindowMode();
    void SwitchMouseWindowMode(tagReadWay NewReadWay,HWND hWndSelectedWindow,BOOL bStopReadingCurrentSentence);

public:
    CScreenReader(HINSTANCE hInstance);
    ~CScreenReader();
    
    BOOL Run();
};

int WINAPI WinMain (HINSTANCE hInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpCmdLine,
                    int nCmdShow
                    )
{
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);
    UNREFERENCED_PARAMETER(nCmdShow);

    CScreenReader ScreenReader(hInstance);
    if (!ScreenReader.Run())
        return -1;

    return 0;
}

BOOL CScreenReader::CallBackEnumVoices(WCHAR* TTSName,PVOID UserParam)
{
    CPopUpMenu* pPopUpMenuVoices = (CPopUpMenu*)UserParam;
#ifdef _DEBUG
    OutputDebugStringW(TTSName);
    OutputDebugStringW(L"\r\n");
#endif

    // add voice to popupmenu
    pPopUpMenuVoices->Add(TTSName);

    return TRUE;
}

void CScreenReader::StartStopReading(BOOL bStart)
{
    this->pOptions->bEnabled = bStart;
    this->pPopUpMenu->SetCheckedState(this->MenuIdEnabled,this->pOptions->bEnabled);

    if (this->pOptions->bEnabled)
    {
        this->pNotifyIcon->SetIcon(this->hInstance,IDI_ICON_SCREEN_READER);
    }
    else
    {
        this->pNotifyIcon->SetIcon(this->hInstance,IDI_ICON_SCREEN_READER_DISABLED);
    }
    // if stop required
    if (!this->pOptions->bEnabled)
    {
        this->pVoice->StopReading();
    }
}


BOOL CScreenReader::NotifyIconCallBack(LPPOINT pCursorPos)
{
    UINT ClickedMenuId;
    ClickedMenuId = this->pPopUpMenu->Show(pCursorPos->x,pCursorPos->y,this->pNotifyIcon->GetAssocatedNotifyIconHwnd(),FALSE,TRUE);
    if (ClickedMenuId)
    {
        if (ClickedMenuId == this->MenuIdRead)
        {
            CReadUI::Show(this->hInstance,this->hWndDialog,this->pVoice);
        }
        else if (ClickedMenuId == this->MenuIdReadClipboard)
        {
            this->ReadClipBoard();
        }
        else if (ClickedMenuId == this->MenuIdAutoReadClipBoardContent)
        {
            BOOL bOldState = this->pPopUpMenu->IsChecked(this->MenuIdAutoReadClipBoardContent);
            this->pOptions->bAutoReadClipBoardContent = !bOldState;
            this->pPopUpMenu->SetCheckedState(this->MenuIdAutoReadClipBoardContent,this->pOptions->bAutoReadClipBoardContent);
        }
        else if (ClickedMenuId == this->MenuIdEnabled)
        {
            BOOL bOldState = this->pPopUpMenu->IsChecked(this->MenuIdEnabled);
            
            this->StartStopReading(!bOldState);

        }
        else if (ClickedMenuId == this->MenuIdReadRole)
        {
            BOOL bOldState = this->pPopUpMenu->IsChecked(this->MenuIdReadRole);
            this->pOptions->bReadRoleSystem = !bOldState;
            this->pPopUpMenu->SetCheckedState(this->MenuIdReadRole,this->pOptions->bReadRoleSystem);
        }
        else if (ClickedMenuId == this->MenuIdVoiceOptions)
        {
            CVoiceOptions::Show(this->hInstance,this->hWndDialog,this->pVoice,this->pOptions);
        }
        else if (ClickedMenuId == this->MenuIdAbout)
        {
            CAbout::Show(this->hInstance,this->hWndDialog,NULL);
        }
        else if ( ClickedMenuId == this->MenuIdMakeADonation)
        {
            this->MakeADonation();
        }
        else if ( ClickedMenuId == this->MenuIdCheckForUpdate)
        {
            this->CheckForUpdate();
        }
        else if (ClickedMenuId == this->MenuIdExit)
        {
            this->Exit();
        }
        else if (ClickedMenuId == this->MenuIdHotKeys)
        {
            for (;;)
            {
                if (CHotKeyUI::Show(this->hInstance,this->hWndDialog,this->pOptions)==1)
                {
                    // ok
                    this->UpdateHotKey();
                    if (this->bHotKeyRegistered)
                        // stop only if hotkey changes were successful
                        break;
                }
                else
                    // cancel
                    break;
            }
        }
        else if (ClickedMenuId == this->MenuIdSelectWindow)
        {
            // BOOL bReadOnlySelectedWindow = this->pPopUpMenu->IsChecked(this->MenuIdSelectWindow);
            HWND hWndSelectedWindow;
            if (CSelectWindowUI::Show(this->hInstance,this->hWndDialog,&hWndSelectedWindow) == IDOK )
                this->SwitchMouseWindowMode(ReadWay_WindowMode,hWndSelectedWindow,TRUE);
            else
                this->SwitchMouseWindowMode(ReadWay_MouseMode,hWndSelectedWindow,TRUE);
        }
        else
        {
            // voice name
            WCHAR VoiceName[CScreenReader_VOICE_NAME_MAX_LEN];
            this->pPopUpMenu->GetText(ClickedMenuId,VoiceName,CScreenReader_VOICE_NAME_MAX_LEN);
            if (this->pVoice->SetVoice(VoiceName))
            {
                // store name into options
                wcscpy(this->pOptions->VoiceName,VoiceName);

                this->pOptions->LoadVoiceSpeedAndVolume();
                this->pVoice->SetSpeedAndVolume(this->pOptions->VoiceSpeed,(USHORT)this->pOptions->VoiceVolume);

                // unselect last voice menu item
                if (this->CurrentVoiceMenuId)
                    this->pPopUpMenu->SetCheckedState(this->CurrentVoiceMenuId,FALSE);

                // select new voice menu item
                this->pPopUpMenu->SetCheckedState(ClickedMenuId,TRUE);

                // store new voice menu item
                this->CurrentVoiceMenuId = ClickedMenuId;
            }
        }
    }

    // return TRUE to restore focus to notify icon, FALSE to keep focus
    return FALSE;
}

void CScreenReader::SwitchMouseWindowMode()
{
    switch (this->ReadWay)
    {
    case ReadWay_WindowMode:
        this->SwitchMouseWindowMode(ReadWay_MouseMode,NULL,TRUE);

        if (this->pOptions->PoolingMouseTime != this->pOptions->PoolingWindowTime)
        {
            ::KillTimer(this->hWndDialog,0);
            ::SetTimer(this->hWndDialog,0,this->pOptions->PoolingMouseTime,CScreenReader::TimerProc);
        }

        break;
    default:
    case ReadWay_MouseMode:
        {
            POINT Point;
            ::GetCursorPos(&Point);
            HWND hWnd = WindowFromPoint( Point );
            hWnd = this->SmallestWindowFromPoint(Point);
            this->SwitchMouseWindowMode(ReadWay_WindowMode,hWnd,TRUE);

            if (this->pOptions->PoolingMouseTime != this->pOptions->PoolingWindowTime)
            {
                ::KillTimer(this->hWndDialog,0);
                ::SetTimer(this->hWndDialog,0,this->pOptions->PoolingWindowTime,CScreenReader::TimerProc);
            }
        }
        break;
    }
}

//-----------------------------------------------------------------------------
// Name: SmallestWindowFromPoint (from PasswordSpy by Brian Friesen)
// Object: Find the smallest window still containing the point
//          WindowFromPoint returns the first window in the Z-order ->
//          if the control is surrounded by a Group Box or some other control,
//          WindowFromPoint returns the handle to the surrounding control instead
//          to the control.
// Parameters : 
//     in : HWND hwnd : target window handle
//          BOOL fDraw : TRUE to draw, FALSE to clear
// Return : TRUE on success
//-----------------------------------------------------------------------------
HWND CScreenReader::SmallestWindowFromPoint( const POINT point )
{    
    RECT rect, rcTemp;
    HWND hParent, hWnd, hTemp;

    hWnd = WindowFromPoint( point );
    if( hWnd != NULL )
    {
        GetWindowRect( hWnd, &rect );
        hParent = GetParent( hWnd );

        // Has window a parent?
        if( hParent != NULL )
        {
            // Search down the Z-Order
            hTemp = hWnd;
            do{
                hTemp = GetWindow( hTemp, GW_HWNDNEXT );
                if (!hTemp)
                    break;
                // Search window contains the point, has the same parent, and is visible?
                GetWindowRect( hTemp, &rcTemp );
                if(PtInRect(&rcTemp, point) && (GetParent(hTemp) == hParent) && IsWindowVisible(hTemp))
                {
                    // Is it smaller?
                    if(((rcTemp.right - rcTemp.left) * (rcTemp.bottom - rcTemp.top)) < ((rect.right - rect.left) * (rect.bottom - rect.top)))
                    {
                        // Found new smaller window!
                        hWnd = hTemp;
                        GetWindowRect(hWnd, &rect);
                    }
                }
            }while( hTemp != NULL );
        }
    }

    return hWnd;
}

void CScreenReader::SwitchMouseWindowMode(tagReadWay NewReadWay,HWND hWndSelectedWindow,BOOL bStopReadingCurrentSentence)
{
    switch (NewReadWay)
    {
    case ReadWay_WindowMode:
        // switch state
        this->pPopUpMenu->SetCheckedState(this->MenuIdSelectWindow,TRUE);

        if (bStopReadingCurrentSentence)
        {
            // stop previous reading if any
            if (this->pVoice)
                this->pVoice->StopReading();
        }

        // switch mode
        this->hWndSelectedWindow = hWndSelectedWindow;
        this->ReadWay = ReadWay_WindowMode;

        break;
    default:
    case ReadWay_MouseMode:
        // Go back to standard mode
        this->pPopUpMenu->SetCheckedState(this->MenuIdSelectWindow,FALSE);

        if (bStopReadingCurrentSentence)
        {
            // stop previous reading if any
            if (this->pVoice)
                this->pVoice->StopReading();
        }

        // switch mode
        this->ReadWay = ReadWay_MouseMode;
        break;
    }
}

void CScreenReader::ReadClipBoard()
{
    ::OpenClipboard(this->hWndDialog);
    WCHAR* pw = (WCHAR*)::GetClipboardData(CF_UNICODETEXT);
    if (pw)
    {
        // when synchronous, 
        //      - bock message pump, 
        //      - block other spying clipboard applications
        //      - block application responsible of message
        // this->pVoice->Read(pw,TRUE,TRUE);

        this->pVoice->Read(pw,TRUE,FALSE);

        this->WaitCurrentReadToComplete = TRUE;
        // keep last read window handle to avoid to read all text again on each copy
        // this->hWndLastReadWindowHandle = (HWND)-1;

        // reset read step to avoid full text reading each time we do a copy
        this->ReadStep=ReadStep_NoContentChangeAllSentencesRead;
    }
#ifdef _DEBUG
    else
        ::MessageBox(NULL, _T("Empty Cliboard"), _T("Error"),MB_OK|MB_ICONERROR);
#endif
    ::CloseClipboard();
}

VOID CALLBACK CScreenReader::TimerProc(HWND hWnd,
                                        UINT uMsg,
                                        UINT_PTR idEvent,
                                        DWORD dwTime
                                        )
{
    UNREFERENCED_PARAMETER(uMsg);
    UNREFERENCED_PARAMETER(idEvent);
    UNREFERENCED_PARAMETER(dwTime);

    CScreenReader* pCScreenReader; 
    pCScreenReader=CScreenReader::GetAssociatedObject(hWnd);
    if (!pCScreenReader)
        return;

    // if read option is enabled 
    if (pCScreenReader->pOptions->bEnabled)
    {
        switch (pCScreenReader->ReadWay)
        {
        default:
        case ReadWay_MouseMode:
            pCScreenReader->ReadAccessibleObject(TRUE);
        	break;
        case ReadWay_WindowMode:
            // if invalid window handle, go back in mouse mode
            if (::IsWindow(pCScreenReader->hWndSelectedWindow))
                pCScreenReader->ReadAccessibleObject(FALSE);
            else
            {
                pCScreenReader->SwitchMouseWindowMode(ReadWay_MouseMode,NULL,TRUE);
                pCScreenReader->ReadAccessibleObject(TRUE);
            }
            break;
        }
    }
}

CScreenReader::CScreenReader(HINSTANCE hInstance)
{
    this->hInstance = hInstance;
    this->pVoice = NULL;
    this->pOptions = NULL;
    this->pNotifyIcon = NULL;
    this->hWndDialog = NULL;
    this->pPopUpMenu = NULL;
    this->pPopUpMenuVoices = NULL;
    this->WaitCurrentReadToComplete = FALSE;
    this->CompleteEvent = NULL;

    this->hWndLastReadWindowHandle=(HWND)-1;
    this->hWndSelectedWindow = NULL;
    this->ReadWay = ReadWay_MouseMode;
    this->LastWindowContent = NULL;
    this->LastWindowContentSize = 0;
    this->ReadStep=ReadStep_NoContentChangeAllSentencesRead;
    this->LastChangeStart = NULL;
    this->LastChangeSize=0;
    this->LastReadStart = NULL;
    this->LastReadSize=0;
    this->LastTextToRead = NULL;

    this->MenuIdRead = 0;
    this->MenuIdReadClipboard = 0;
    this->MenuIdEnabled = 0;
    this->MenuIdReadRole = 0;
    this->MenuIdAbout = 0;
    this->MenuIdMakeADonation = 0;
    this->MenuIdCheckForUpdate = 0;
    this->MenuIdExit = 0;
    this->CurrentVoiceMenuId = 0;
    this->MenuIdHotKeys = 0;
    this->MenuIdSelectWindow = 0;

    this->EndSessionProcessed = FALSE;
    
    this->hModuleUser32 = ::LoadLibrary(_T("User32.dll"));
    this->pShutdownBlockReasonCreate = (pfShutdownBlockReasonCreate)::GetProcAddress(this->hModuleUser32,"ShutdownBlockReasonCreate");
    this->pShutdownBlockReasonDestroy = (pfShutdownBlockReasonDestroy)::GetProcAddress(this->hModuleUser32,"ShutdownBlockReasonDestroy");
}
CScreenReader::~CScreenReader()
{
    if (this->LastTextToRead)
        free(this->LastTextToRead);

    if (this->LastWindowContent)
        free(this->LastWindowContent);

    if (this->hModuleUser32)
        ::FreeLibrary(this->hModuleUser32);
}

BOOL CScreenReader::Run()
{
    if (FAILED(::CoInitialize(NULL)))
        return FALSE;

    this->pVoice = new CVoice();// must be after ::CoInitialize();
    if (!this->pVoice->Load())
    {
        // sapi 5 not found
        MessageBox(NULL,_T("This software require Microsoft Speech API 5\r\n"),_T("Error"),MB_OK|MB_ICONERROR);
        return FALSE;
    }

    // load options
    this->pOptions = new COptions();
    this->pOptions->Load();

    // create notify icon
    UINT IdIcon = IDI_ICON_SCREEN_READER;
    if (!this->pOptions->bEnabled)
        IdIcon = IDI_ICON_SCREEN_READER_DISABLED;
    HICON hIcon = CDialogHelper::LoadIcon(this->hInstance,IdIcon,16,16);
    this->pNotifyIcon = new CNotifyIcon(this->hInstance,hIcon,_T("Screen Reader"),this);

    // create popup menu associated to notify icon
    this->pPopUpMenu = new CPopUpMenu();

    // add all known voices to menu
    this->pPopUpMenuVoices = new CPopUpMenu(this->pPopUpMenu);

    this->MenuIdAbout = this->pPopUpMenu->Add(_T("About"),IDI_ICON_ABOUT,this->hInstance);
    this->MenuIdMakeADonation = this->pPopUpMenu->Add(_T("Make A Donation"),IDI_ICON_DONATION,this->hInstance);
    this->MenuIdCheckForUpdate = this->pPopUpMenu->Add(_T("Check For Update"),IDI_ICON_UPDATE,this->hInstance);
    this->pPopUpMenu->AddSeparator();

    this->MenuIdHotKeys = this->pPopUpMenu->Add(_T("Hot Keys"),IDI_ICON_HOTKEYS,this->hInstance);

    this->pPopUpMenu->AddSeparator();
    this->MenuIdEnabled = this->pPopUpMenu->Add(_T("Enabled"));
    this->MenuIdReadRole = this->pPopUpMenu->Add(_T("Read Role"));
    this->pPopUpMenu->AddSubMenu(_T("Voices"),this->pPopUpMenuVoices,IDI_ICON_VOICES,this->hInstance);
    this->pVoice->EnumVoices(CallBackEnumVoices,this->pPopUpMenuVoices);
    this->MenuIdVoiceOptions = this->pPopUpMenu->Add(_T("Voice Options"),IDI_ICON_VOICES_OPTIONS,this->hInstance);

    this->pPopUpMenu->AddSeparator();
    this->MenuIdSelectWindow = this->pPopUpMenu->Add(_T("Read Only Selected Window Content"),IDI_ICON_WINDOW,this->hInstance);

    this->pPopUpMenu->AddSeparator();
    this->MenuIdRead = this->pPopUpMenu->Add(_T("Read"),IDI_ICON_READ,this->hInstance);
    this->MenuIdReadClipboard = this->pPopUpMenu->Add(_T("Read ClipBoard"),IDI_ICON_READ_CLIPBOARD,this->hInstance);

    this->MenuIdAutoReadClipBoardContent = this->pPopUpMenu->Add(_T("Auto Read ClipBoard On Change"));
    if (this->pOptions->bAutoReadClipBoardContent)
        this->pPopUpMenu->SetCheckedState(this->MenuIdAutoReadClipBoardContent,TRUE);

    this->pPopUpMenu->AddSeparator();
    this->MenuIdExit = this->pPopUpMenu->Add(_T("Exit"),IDI_ICON_EXIT,this->hInstance);

    // adjust popupmenu, notify icon, and TTS engine according to options
    if (this->pOptions->bEnabled)
        this->pPopUpMenu->SetCheckedState(this->MenuIdEnabled,TRUE);

    if (this->pOptions->bReadRoleSystem)
        this->pPopUpMenu->SetCheckedState(this->MenuIdReadRole,TRUE);

    // loop though voice menu to check used voice
    int CurrentMenuId;
    WCHAR VoiceName[CScreenReader_VOICE_NAME_MAX_LEN];
    for (int Cnt = 0; Cnt < this->pPopUpMenuVoices->GetItemCount(); Cnt++)
    {
        CurrentMenuId = this->pPopUpMenuVoices->GetID(Cnt);
        this->pPopUpMenuVoices->GetText(CurrentMenuId,VoiceName,CScreenReader_VOICE_NAME_MAX_LEN);
        // if voice name matchs
        if (wcsicmp(VoiceName,this->pOptions->VoiceName) == 0)
        {
            // check voice name menu
            this->pPopUpMenu->SetCheckedState(CurrentMenuId,TRUE);

            // store selected menu id
            this->CurrentVoiceMenuId = CurrentMenuId;

            // load specified voice
            this->pVoice->SetVoice(VoiceName);

            break;
        }
    }
    // set voice volume and speed
    this->pVoice->SetSpeedAndVolume(this->pOptions->VoiceSpeed,(USHORT)this->pOptions->VoiceVolume);
    this->CompleteEvent = this->pVoice->GetSpeakCompleteEvent();

    // display notify icon
    this->pNotifyIcon->Show();

    // wait exit message
    this->WaitApplicationEnd();
    
    // destroy object after reading thread end

    delete pNotifyIcon;

    delete this->pPopUpMenuVoices;
    this->pPopUpMenuVoices = NULL;

    delete this->pPopUpMenu;
    this->pPopUpMenu = NULL;

    delete this->pVoice; // must be before ::CoUninitialize();
    this->pVoice=NULL;

    delete this->pOptions;
    this->pOptions = NULL;

    ::CoUninitialize();
    return TRUE;
}


void CScreenReader::MakeADonation()
{
    BOOL bEuroCurrency=FALSE;
    HKEY hKey;
    wchar_t pszCurrency[2];
    DWORD Size=2*sizeof(wchar_t);
    memset(pszCurrency,0,Size);
    TCHAR pszMsg[3*MAX_PATH];

    // check that HKEY_CURRENT_USER\Control Panel\International\sCurrency contains the euro symbol
    // retrieve it in unicode to be quite sure of the money symbol
    if (::RegOpenKeyEx(HKEY_CURRENT_USER,_T("Control Panel\\International"),0,KEY_QUERY_VALUE,&hKey)==ERROR_SUCCESS)
    {
        // use unicode version only to make string compare
        if (::RegQueryValueExW(hKey,L"sCurrency",NULL,NULL,(LPBYTE)pszCurrency,&Size)==ERROR_SUCCESS)
        {
            if (wcscmp(pszCurrency,L"")==0)
                bEuroCurrency=TRUE;
        }
        // close open key
        ::RegCloseKey(hKey);
    }
    // yes, you can do it if u don't like freeware and open sources soft
    // but if you make it, don't blame me for not releasing sources anymore
    _tcscpy(pszMsg,_T("https://www.paypal.com/cgi-bin/webscr?cmd=_xclick&business=jacquelin.potier@free.fr")
        _T("&item_name=Donation%20for%20ScreenReader&return=http://jacquelin.potier.free.fr/screenreader/"));
    // in case of euro monetary symbol
    if (bEuroCurrency)
        // add it to link
        _tcscat(pszMsg,_T("&currency_code=EUR"));

    // open donation web page
    if (((int)ShellExecute(NULL,_T("open"),pszMsg,NULL,NULL,SW_SHOWNORMAL))<33)
        // display error msg in case of failure
        ::MessageBox(NULL,_T("Error Opening default browser. You can make a donation going to ")
        _T("http://jacquelin.potier.free.fr"),
        _T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
}
void CScreenReader::CheckForUpdate()
{
    CSoftwareUpdate::CheckForUpdate(ScreenReader_PAD_Url,ScreenReaderVersionSignificantDigits,ScreenReaderDownloadLink);
}
/*
WCHAR* __fastcall GetNextSentenceForSplitter(WCHAR* Text,WCHAR c)
{
    WCHAR NextChar;
    WCHAR* psz;
    psz = wcschr(Text,c);
    if (psz)
    {
        NextChar = *(psz+1);
        if ( (NextChar == ' ') || (NextChar == '\r') || (NextChar == '\n')  || (NextChar == '\t') )
            return psz;
    }
    return NULL;
}

WCHAR* __fastcall GetPreviousSentenceForSplitter(WCHAR* Text,SIZE_T From,WCHAR c)
{
    WCHAR NextChar;
    WCHAR* psz;
    WCHAR* LastResult=NULL;
    WCHAR* NextSearch;
    
    for(NextSearch=Text;;)
    {
        // find next matching char
        psz = wcschr(NextSearch,c);
        if (!psz)
            break;
        if ( psz>(Text+From) )
            break;

        NextSearch = psz+1;
        NextChar = *NextSearch;
        if ( (NextChar == ' ') || (NextChar == '\r') || (NextChar == '\n')  || (NextChar == '\t') )
            LastResult =psz;
    }

    return LastResult;
}
*/


/***
*int FindNumberOfIdenticalCharsFromBegin(first, last, count) - compare first count chars of wchar_t strings
*
*Purpose:
*       Compares two strings for lexical order.  The comparison stops
*       after: (1) a difference between the strings is found, (2) the end
*       of the strings is reached, or (3) count characters have been
*       compared (wide-character strings).
*
*Entry:
*       wchar_t *first, *last - strings to compare
*       size_t count - maximum number of characters to compare
*
*Exit:
*       returns number of identical char
*Exceptions:
*
*******************************************************************************/
SIZE_T FindNumberOfIdenticalCharsFromBegin (
                                            const wchar_t * first,
                                            const wchar_t * last,
                                            size_t count,
                                            BOOL InsensitiveSearch
                                            )
{
    if (!count)
        return 0;
    size_t initial_count;
    wchar_t f;
    wchar_t l;


    initial_count = count;
    do
    {
        if (InsensitiveSearch)
        {
            f = towlower(*first);
            l = towlower(*last);
        }
        else
        {
            f=*first;
            l=*last;
        }

        if ( (f != l) || (!f) )
            break;

        first++;
        last++;
        count--;
    }while( count );

    return initial_count-count;
}

/***
*int FindNumberOfIdenticalCharsFromBegin(first, last, count) - compare first count chars of wchar_t strings
*
*Purpose:
*       Compares two strings for lexical order.  The comparison stops
*       after: (1) a difference between the strings is found, (2) the begin
*       of the strings is reached, or (3) count characters have been
*       compared (wide-character strings).
*
*Entry:
*       wchar_t *first, *last - strings to compare
*       size_t count - maximum number of characters to compare
*
*Exit:
*       returns number of identical char
*Exceptions:
*
*******************************************************************************/
SSIZE_T FindNumberOfIdenticalCharsFromEnd (
                                        const wchar_t * first,
                                        const wchar_t * last,
                                        size_t count,
                                        BOOL InsensitiveSearch
                                        )
{
    if (!count)
        return 0;
    size_t initial_count;
    const wchar_t * initial_first=first;
    const wchar_t * initial_last=last;
    wchar_t f;
    wchar_t l;

    // go to end of each string
    while (*first)
    {
        first++;
    }
    first--;// position before '\0'
    while (*last)
    {
        last++;
    }
    last--;// position before '\0'

    initial_count = count;
    do 
    {
        if (InsensitiveSearch)
        {
            f = towlower(*first);
            l = towlower(*last);
        }
        else
        {
            f=*first;
            l=*last;
        }

        if ( (f != l) || (first<initial_first) || (last<initial_last) )
            break;

        first--;
        last--;
        count--;
    }while ( count );

    return initial_count-count;
}

// avoid displayed text to be replaced by link content 
// use NewWindowContent instead of ObjectTextValue, for 
// ObjectTextValue beginning with the following key words
WCHAR* RemovedStartingTextValues[]=
{
    L"http://",
    L"https://",
    L"javascript:",
    L"mailto://",
    L"ftp://",
    L"file://",
    L"res://",
};

BOOL CScreenReader::GetChangePosition(WCHAR* NewWindowContent,WCHAR** pChangeStart,SIZE_T* pChangeSize)
{
    // initialize output vars
    *pChangeStart = NewWindowContent;
    *pChangeSize = 0;
    SIZE_T NewWindowContentSize = wcslen(NewWindowContent);
    SIZE_T NbCharToCheck = __min(NewWindowContentSize,this->LastWindowContentSize);

    // if previous or current read is empty
    if (NbCharToCheck == 0)
        return TRUE;

    SIZE_T NbIdenticalCharsAtBegin;
    SIZE_T NbIdenticalCharsAtEnd;

    NbIdenticalCharsAtBegin = FindNumberOfIdenticalCharsFromBegin(NewWindowContent,this->LastWindowContent,NbCharToCheck,TRUE);
    *pChangeStart = NewWindowContent+NbIdenticalCharsAtBegin;

    if ( NbIdenticalCharsAtBegin==NbCharToCheck )
    {
        // if some text has been added at the end
        if (NewWindowContentSize>NbCharToCheck)
            *pChangeSize = NewWindowContentSize - NbCharToCheck;
        else // end of text has been removed
            *pChangeSize = 0;
        return TRUE;
    }

    // string is modified inside text (begin or middle) --> find if ending char are identical
    NbIdenticalCharsAtEnd = FindNumberOfIdenticalCharsFromEnd(*pChangeStart,this->LastWindowContent+NbIdenticalCharsAtBegin,NbCharToCheck-NbIdenticalCharsAtBegin,TRUE);
    *pChangeSize = NewWindowContentSize-NbIdenticalCharsAtBegin-NbIdenticalCharsAtEnd;

    return TRUE;
}

WCHAR* wcsrpbrkLike(WCHAR* StartingChar,WCHAR* EndingChar,WCHAR* SearchedChars)
{
    if (EndingChar>StartingChar)
        EndingChar--;
    else
        return StartingChar;

    WCHAR* pCurrentChar;
    WCHAR CurrentChar;
    WCHAR* pSearchedChar;
    // for each char from EndingChar to StartingChar
    for (pCurrentChar=EndingChar;pCurrentChar>=StartingChar;pCurrentChar--)
    {
        CurrentChar=*pCurrentChar;
        // for each char in SearchedChars
        for (pSearchedChar = SearchedChars;*pSearchedChar;pSearchedChar++)
        {
            // if we found item
            if (*pSearchedChar == CurrentChar)
                // return pointer to item
                return pCurrentChar;
        }
    }
    // not found
    return NULL;
}
/*
WCHAR* FindNextPunctOrSpacer(WCHAR* StartingChar)
{
    WCHAR* pCurrentChar;
    WCHAR CurrentChar;
    // for each char from EndingChar to StartingChar
    for (pCurrentChar=StartingChar;*pCurrentChar;pCurrentChar++)
    {
        CurrentChar=*pCurrentChar;
        if (   iswspace(CurrentChar) 
            // || iswpunct(CurrentChar) ispunct is too permissive for current use
            || wcschr(CScreenReader_ENDING_SENTENCE_CHARS_W,CurrentChar)
            )
            return pCurrentChar;
    }
    // not found
    return NULL;
}

WCHAR* FindPreviousPunctOrSpacer(WCHAR* StartingChar,WCHAR* EndingChar)
{
    if (EndingChar>StartingChar)
        EndingChar--;
    else
        return StartingChar;

    WCHAR* pCurrentChar;
    WCHAR CurrentChar;
    // for each char from EndingChar to StartingChar
    for (pCurrentChar=EndingChar;pCurrentChar>=StartingChar;pCurrentChar--)
    {
        CurrentChar=*pCurrentChar;
        if (   iswspace(CurrentChar) 
            // || iswpunct(CurrentChar) ispunct is too permissive for current use
            || wcschr(CScreenReader_ENDING_SENTENCE_CHARS_W,CurrentChar)
            )
            return pCurrentChar;
    }
    // not found
    return NULL;
}
*/
BOOL HasNonAlphaNum(WCHAR* StartingChar, SIZE_T Size)
{
    WCHAR* pCurrentChar;
    WCHAR* pEndChar;
    WCHAR CurrentChar;
    pEndChar = StartingChar+Size;
    // for each char from EndingChar to StartingChar
    for (pCurrentChar=StartingChar;(*pCurrentChar) && (pCurrentChar<pEndChar);pCurrentChar++)
    {
        CurrentChar=*pCurrentChar;
        if ( !iswalnum(CurrentChar) )
            return TRUE;
    }
    // not found
    return FALSE;
}

BOOL IsTextToReadTheSame(WCHAR* String1,WCHAR* String2)
{
    WCHAR* pCh1;
    WCHAR* pCh2;
    WCHAR* StringToCheck;

    // loop until end of String1 or String2
    for (pCh1=String1,pCh2=String2;
         (*pCh1) && (*pCh2);
         pCh1++,pCh2++
         )
    {
        // inside string there is a change
        if (*pCh1 != *pCh2)
            return FALSE;
    }
    
    if (pCh1==NULL)
    {
        // is string are identical
        if (pCh2 == NULL)
            return TRUE;
        else
            StringToCheck = pCh2;
    }
    else
        StringToCheck = pCh1;

    // assume that end of string only contains non alphanum chars (not readable chars)
    for (pCh1 = StringToCheck; *pCh1; pCh1++)
    {
        if (iswalnum(*pCh1))
            return FALSE;
    }
    // at this point the longest string contains only non readable char at the end compared to the shortest string
    // --> concider that string are the same
    return TRUE;
}

WCHAR* FindNextNonAlphaNum(WCHAR* StartingChar)
{
    WCHAR* pCurrentChar;
    WCHAR CurrentChar;
    // for each char from EndingChar to StartingChar
    for (pCurrentChar=StartingChar;*pCurrentChar;pCurrentChar++)
    {
        CurrentChar=*pCurrentChar;
        if ( !iswalnum(CurrentChar) )
            return pCurrentChar;
    }
    // not found
    return NULL;
}

WCHAR* FindPreviousNonAlphaNum(WCHAR* StartingChar,WCHAR* EndingChar)
{
    if (EndingChar>StartingChar)
        EndingChar--;
    else
        return StartingChar;

    WCHAR* pCurrentChar;
    WCHAR CurrentChar;
    // for each char from EndingChar to StartingChar
    for (pCurrentChar=EndingChar;pCurrentChar>=StartingChar;pCurrentChar--)
    {
        CurrentChar=*pCurrentChar;
        if ( !iswalnum(CurrentChar) )
            return pCurrentChar;
    }
    // not found
    return NULL;
}

WCHAR* FindNextAlphaNum(WCHAR* StartingChar)
{
    WCHAR* pCurrentChar;
    WCHAR CurrentChar;
    // for each char from EndingChar to StartingChar
    for (pCurrentChar=StartingChar;*pCurrentChar;pCurrentChar++)
    {
        CurrentChar=*pCurrentChar;
        if ( iswalnum(CurrentChar) )
            return pCurrentChar;
    }
    // not found
    return NULL;
}

WCHAR* FindPreviousAlphaNum(WCHAR* StartingChar,WCHAR* EndingChar)
{
    if (EndingChar>StartingChar)
        EndingChar--;
    else
        return StartingChar;

    WCHAR* pCurrentChar;
    WCHAR CurrentChar;
    // for each char from EndingChar to StartingChar
    for (pCurrentChar=EndingChar;pCurrentChar>=StartingChar;pCurrentChar--)
    {
        CurrentChar=*pCurrentChar;
        if ( iswalnum(CurrentChar) )
            return pCurrentChar;
    }
    // not found
    return NULL;
}

BOOL CScreenReader::UpdateReadBufferToReadCurrentOrPreviousWord()
{
    // in case EndingChar is splitter, space, ...
    WCHAR* EndOfWord;
    if (iswalnum(*this->LastReadStart))
        EndOfWord = this->LastReadStart;
    else
        EndOfWord = FindPreviousAlphaNum(this->LastWindowContent,this->LastReadStart);
    if (!EndOfWord) // if only spacer before current change
        // let this->LastReadStart && this->LastReadSize unchanged
        return TRUE;

    WCHAR* BeginOfWord;
    BeginOfWord = FindPreviousNonAlphaNum(this->LastWindowContent,EndOfWord);
    if (BeginOfWord)
        BeginOfWord++;
    else
        BeginOfWord = this->LastWindowContent;

    EndOfWord = FindNextNonAlphaNum(BeginOfWord);
    if (!EndOfWord)
        EndOfWord = BeginOfWord+wcslen(BeginOfWord);

    this->LastReadStart = BeginOfWord;
    this->LastReadSize = EndOfWord-BeginOfWord;

    return TRUE;
}
BOOL CScreenReader::UpdateReadBufferToReadCurrentOrPreviousSentence()
{
    WCHAR* BeginOfSequence;

    BeginOfSequence = wcsrpbrkLike(this->LastWindowContent,this->LastReadStart,CScreenReader_ENDING_SENTENCE_CHARS_W);
    if (BeginOfSequence)
    {
        // if a splitter has been found
        if (BeginOfSequence != this->LastWindowContent) // aka if (wcschr(CScreenReader_ENDING_SENTENCE_CHARS_W,*BeginOfSequence))
            // go after splitter
            BeginOfSequence++;
    }
    else
        BeginOfSequence = this->LastWindowContent;

    WCHAR* EndOfSentence;
    // find next splitter
    EndOfSentence = wcspbrk(this->LastReadStart,CScreenReader_ENDING_SENTENCE_CHARS_W);
    if (EndOfSentence)
    {
        // go after ending char
        EndOfSentence++;

        // manage the case of '...' '\r\n', '!!!' --> get the very last of CScreenReader_ENDING_SENTENCE_CHARS_W
        while(wcschr(CScreenReader_ENDING_SENTENCE_CHARS_W,*EndOfSentence))
        {
            // go after ending char
            EndOfSentence++;
        }

    }
    else
        EndOfSentence = this->LastReadStart+wcslen(this->LastReadStart);


    this->LastReadStart = BeginOfSequence;
    this->LastReadSize = EndOfSentence-BeginOfSequence;

    return TRUE;
}


BOOL CScreenReader::UpdateReadBufferToReadCurrentOrPreviousBlock()
{
    WCHAR* BeginOfBlock;

    BeginOfBlock = wcsrpbrkLike(this->LastWindowContent,this->LastReadStart,CScreenReader_ENDING_BLOCKS_CHARS_W);
    if (BeginOfBlock)
    {
        // if a splitter has been found
        if (BeginOfBlock != this->LastWindowContent) // aka if (wcschr(CScreenReader_ENDING_SENTENCE_CHARS_W,*BeginOfBlock))
            // go after splitter
            BeginOfBlock++;
    }
    else
        BeginOfBlock = this->LastWindowContent;

    WCHAR* EndOfBlock;
    // find next splitter
    EndOfBlock = wcspbrk(this->LastReadStart,CScreenReader_ENDING_BLOCKS_CHARS_W);
    if (EndOfBlock)
    {
        // go after ending char
        EndOfBlock++;

        // manage the case of '...' '\r\n', '!!!' --> get the very last of CScreenReader_ENDING_SENTENCE_CHARS_W
        while(wcschr(CScreenReader_ENDING_BLOCKS_CHARS_W,*EndOfBlock))
        {
            // go after ending char
            EndOfBlock++;
        }

    }
    else
        EndOfBlock = this->LastReadStart+wcslen(this->LastReadStart);


    this->LastReadStart = BeginOfBlock;
    this->LastReadSize = EndOfBlock-BeginOfBlock;

    return TRUE;
}

BOOL CScreenReader::UpdateReadBufferToReadEndOfSentence()
{
    WCHAR* EndOfSentence;
    // find next splitter
    EndOfSentence = wcspbrk(this->LastReadStart,CScreenReader_ENDING_SENTENCE_CHARS_W);
    if (EndOfSentence)
    {
        // go after ending char
        EndOfSentence++;

        // manage the case of '...' '\r\n', '!!!' --> get the very last of CScreenReader_ENDING_SENTENCE_CHARS_W
        while(wcschr(CScreenReader_ENDING_SENTENCE_CHARS_W,*EndOfSentence))
        {
            // go after ending char
            EndOfSentence++;
        }

        this->LastReadSize = EndOfSentence-this->LastReadStart;
    }
    else
    {
        this->LastReadSize = wcslen(this->LastReadStart);
    }
    return TRUE;
}

// if EndingChar !iswalnum, get previous word, else get current word
WCHAR* GetCurrentOrPreviousWordStart(WCHAR* StartingChar,WCHAR* EndingChar)
{
    if (EndingChar<=StartingChar)
        return StartingChar;

    WCHAR* EndOfWord;
    if ( iswalnum(*EndingChar) )
    {
        EndOfWord = FindNextNonAlphaNum(EndingChar);
        if (EndOfWord == NULL)
            EndOfWord = EndingChar+wcslen(EndingChar)-1;
    }
    else
    {
        EndOfWord = FindPreviousAlphaNum(StartingChar,EndingChar);
        if (!EndOfWord)
            return EndingChar;
    }

    WCHAR* BeginOfWord;
    BeginOfWord = FindPreviousNonAlphaNum(StartingChar,EndOfWord);
    if (BeginOfWord)
        BeginOfWord++;
    else
        BeginOfWord = StartingChar;
    return BeginOfWord;
}

// if EndingChar is an ENDING_BLOCKS_CHARS_W, get previous block, else get current block
WCHAR* GetCurrentOrPreviousBlockStart(WCHAR* StartingChar,WCHAR* EndingChar)
{
    WCHAR* pCurrentChar;
    WCHAR CurrentChar;
    if (EndingChar<=StartingChar)
        return StartingChar;

    BOOL ValidCharFound = FALSE; // assume to get a block with content (case of "...", "!!!" and so on)

    // for each char from EndingChar to StartingChar
    for (pCurrentChar=EndingChar;pCurrentChar>=StartingChar;pCurrentChar--)
    {
        CurrentChar=*pCurrentChar;
        if ( wcschr(CScreenReader_ENDING_BLOCKS_CHARS_W,CurrentChar) )
        {
            if (ValidCharFound)
                return pCurrentChar+1;
        }
        else
            ValidCharFound = TRUE;
    }
    return StartingChar;
}


BOOL CScreenReader::ReadAccessibleObject(BOOL bFromMouse)
{
    BOOL bSameWindowHandleAsLastRead=TRUE;
    BOOL bSuccess = FALSE;
    WCHAR* NewWindowContent=NULL; // ObjectName
    WCHAR* ObjectTextRole=NULL;
    WCHAR* ObjectTextValue=NULL;
    WCHAR* psz = NULL;
    WCHAR* TextToRead = NULL;

    // if voice is not loaded
    if (!this->pVoice->IsLoaded())
        return FALSE;

    if (this->WaitCurrentReadToComplete)
    {
        if (WaitForSingleObject(this->CompleteEvent,0) != WAIT_OBJECT_0)
            return FALSE;

        this->WaitCurrentReadToComplete = FALSE;
    }
        

    // if text was successfully retrieved
    if (bFromMouse)
    {
        bSuccess = CObjectNameFromMouse::GetObjectNameFromMouse(&NewWindowContent,&ObjectTextRole,&ObjectTextValue);
        POINT Point;
        ::GetCursorPos(&Point);
        HWND hWnd = WindowFromPoint( Point );
        hWnd = this->SmallestWindowFromPoint(Point);
        bSameWindowHandleAsLastRead = (hWnd == this->hWndLastReadWindowHandle);
        this->hWndLastReadWindowHandle = hWnd;
    }
    else
    {
        bSameWindowHandleAsLastRead = (this->hWndLastReadWindowHandle == this->hWndSelectedWindow);
        this->hWndLastReadWindowHandle = this->hWndSelectedWindow;
        bSuccess = CObjectNameFromMouse::GetObjectNameFromWindow(this->hWndSelectedWindow,&NewWindowContent,&ObjectTextRole,&ObjectTextValue);
    }
    if (!bSuccess)
        goto CleanUp;

    // if ObjectTextValue is defined, try to use it instead of ObjectName
    if (ObjectTextValue)
    {
        if (NewWindowContent)
        {
            BOOL bExchangeNameAndValue=FALSE;

            // remove digit only values
            SIZE_T ObjectTextValueLen = wcslen(ObjectTextValue);

            if (ObjectTextValueLen>0)
            {
                BOOL bOnlyDigits = TRUE;
                SIZE_T Cnt;
                for (Cnt=0;Cnt<ObjectTextValueLen;Cnt++)
                {
                    if(!::iswdigit(ObjectTextValue[Cnt]))
                    {
                        bOnlyDigits=FALSE;
                        break;
                    }
                }

                bExchangeNameAndValue = !bOnlyDigits;

                // remove link value (http,https,javascript:)
                if (bExchangeNameAndValue)
                {
                    WCHAR* KeyWord;
                    for (Cnt=0;Cnt<_countof(RemovedStartingTextValues);Cnt++)
                    {
                        KeyWord = RemovedStartingTextValues[Cnt];
                        if (wcsnicmp(ObjectTextValue,KeyWord,wcslen(KeyWord))==0)
                        {
                            bExchangeNameAndValue=FALSE;
                            break;
                        }
                    }
                }
            }


            if (bExchangeNameAndValue)
            {
#ifdef _DEBUG
                OutputDebugStringW(NewWindowContent);
                OutputDebugStringW(L" replaced by ");
                OutputDebugStringW(ObjectTextValue);
                OutputDebugStringW(L"\r\n");
#endif
                free(NewWindowContent);
                NewWindowContent = ObjectTextValue;
                ObjectTextValue = NULL;
            }
            else
            {
                free(ObjectTextValue);
                ObjectTextValue = NULL;
            }
        }
        else
        {
            NewWindowContent = ObjectTextValue;
            ObjectTextValue = NULL;
        }
    }

    // assume there is something to read
    if (!NewWindowContent)
        goto CleanUp;

    ////////////////////////////////////////////////////////////////////////////////////
    // window content is in NewWindowContent
    ////////////////////////////////////////////////////////////////////////////////////

    BOOL bPurgeBeforeRead = TRUE;
    BOOL bContentHasChanged = TRUE;
    BOOL bFullNewChange = TRUE;
    WCHAR* NewChangeStart;
    SIZE_T NewChangeSize;

    if ( (!bSameWindowHandleAsLastRead) || (this->LastWindowContent==NULL) )
    {
        // clear that is currently being read
        bPurgeBeforeRead = TRUE;

        // next step will be to read the next sentence
        this->ReadStep=ReadStep_NoContentChangeReadNextSentence;

        // full new content, don't try to follow changes
        this->LastChangeStart = NULL;
        this->LastChangeSize=0;

        // ask to read the first sentence
        this->LastReadStart = NewWindowContent;
        this->LastReadSize=0;
        // UpdateReadBufferToReadEndOfSentence is done in case ReadStep_NoContentChangeReadNextSentence:
        // this->UpdateReadBufferToReadEndOfSentence();
    }
    else
    // if (bSameWindowHandleAsLastRead && this->LastWindowContent)
    {
        // let insensitive compare in case of case change in text editor
        bContentHasChanged = (wcsicmp(this->LastWindowContent , NewWindowContent) != 0);
        BOOL bIsReadFinished = !this->pVoice->IsReading();

        // if no change and read is not completed
        if ( (!bContentHasChanged) && (!bIsReadFinished) ) // implies bIsReadFinished when !bContentHasChanged
        {
            // do nothing and wait read has completed
            bSuccess = TRUE;
            goto CleanUp;
        }

        if (bContentHasChanged)
        {
            BOOL bUpdateReadContent = TRUE;
            this->GetChangePosition(NewWindowContent,&NewChangeStart,&NewChangeSize);
                             
            if (this->LastChangeStart)
            {
                SIZE_T LastChangeStartPosition;
                SIZE_T LastReadStartPosition;
                SIZE_T NewChangeStartPosition;
                SIZE_T MergedChangeStartPosition;
                SIZE_T MergedChangeEndPosition;
                SIZE_T MergedChangeSize;
                NewChangeStartPosition = NewChangeStart-NewWindowContent;
                LastReadStartPosition = this->LastReadStart - this->LastWindowContent;
                LastChangeStartPosition = this->LastChangeStart - this->LastWindowContent;

                // if change belong to the last changed area, or is contiguous to changed area
                if ( (LastChangeStartPosition<=(NewChangeStartPosition+NewChangeSize) ) && ( NewChangeStartPosition<=(LastChangeStartPosition+this->LastChangeSize) ) )
                {
                    bFullNewChange = FALSE;

                    // update the changed area : the changed zone is the maximum changed area
                    MergedChangeStartPosition = // NewChangeStartPosition; 
                                                __min(NewChangeStartPosition,LastReadStartPosition);
                    MergedChangeEndPosition = __max(NewChangeStartPosition+NewChangeSize,LastChangeStartPosition+this->LastChangeSize);
                    MergedChangeSize = MergedChangeEndPosition-MergedChangeStartPosition+1;

                    this->LastChangeStart = NewWindowContent+MergedChangeStartPosition;
                    this->LastChangeSize = __min(MergedChangeSize, wcslen(this->LastChangeStart) );// in case of text reduction

                    // if change is after currently read content, we don't need to purge currently spoken text
                    if (NewChangeStartPosition > (LastReadStartPosition+this->LastReadSize))
                    {
                        bPurgeBeforeRead = FALSE;
                        bUpdateReadContent = FALSE;

                        // make LastReadStart point inside the new buffer
                        this->LastReadStart = NewWindowContent + (this->LastReadStart - this->LastWindowContent);

                        // let LastReadSize unchanged
                        // this->LastReadSize = 

                        // force next step to read the next sentence (assume to read in case previous step was in ReadStep_ContentModifiedAllChangedSentencesRead state)
                        this->ReadStep=ReadStep_NoContentChangeReadNextSentence;
                    }
                    //else
                    //{
                    //    // let bUpdateReadContent action do is job
                    //}
                }
                else
                {
                    this->LastChangeStart = NewChangeStart;
                    this->LastChangeSize = NewChangeSize;
                }
            }
            else
            {
                this->LastChangeStart = NewChangeStart;
                this->LastChangeSize = NewChangeSize;
            }

            if (bUpdateReadContent)
            {
                this->LastReadStart = NewChangeStart;
                this->LastReadSize = 0;

                //if remove operation
                if (NewChangeSize == 0)
                {
                    if (NewChangeStart != NewWindowContent)
                        NewChangeStart--;
                    WCHAR ChangedChar = *NewChangeStart;
                    if (iswalpha(ChangedChar))
                    {
                        this->ReadStep = ReadStep_ContentModifiedReadWord;
                        this->LastReadStart = GetCurrentOrPreviousWordStart(NewWindowContent,NewChangeStart);
                    }
                    // else : word not affected --> text already read --> let in current step
                }
                // if single letter changed
                else if (NewChangeSize == 1)
                {
                    WCHAR ChangedChar = *NewChangeStart;
                    
                    if (!iswalpha(ChangedChar))
                    {
                        // if previous char is alpha, read previous word, else (non alpha) previous word could have already been read
                        if (bFullNewChange 
                            || ( (NewChangeStart != NewWindowContent) && ( iswalpha( *(NewChangeStart-1) ) ) )
                            )
                        {
                            this->ReadStep = ReadStep_ContentModifiedReadWord;
                            this->LastReadStart = GetCurrentOrPreviousWordStart(NewWindowContent,NewChangeStart);
                        }
                        // else : text already read --> let in current step
                    }
                    else
                        this->ReadStep = ReadStep_ContentModifiedReadLetter;
                }
                else // if multiple letters changed
                {
                    // check if change occurs inside a single word
                    BOOL bMultipleWordsChange = HasNonAlphaNum(NewChangeStart,NewChangeSize); // all ENDING_BLOCKS_CHARS_W are contained into !isalphanum

                    // in case of multiple words
                    if (bMultipleWordsChange)
                    {
                        // if content include splitter, read current affected sentence --> position read text from previous CScreenReader_ENDING_BLOCKS_CHARS_W
                        this->ReadStep = ReadStep_ContentModifiedReadBlock;
                        this->LastReadStart = GetCurrentOrPreviousBlockStart(NewWindowContent,NewChangeStart);
                    }
                    else
                    {
                        // read only affected word first
                        this->ReadStep = ReadStep_ContentModifiedReadWord;
                        this->LastReadStart = GetCurrentOrPreviousWordStart(NewWindowContent,NewChangeStart);
                    }
                }
            }
        }
    }

    if (bContentHasChanged)
    {
        // store full text size in this->LastWindowContentSize (not the trunked part)
        this->LastWindowContentSize = wcslen(NewWindowContent);

        if (this->LastWindowContent)
            // free previous allocation
            free(this->LastWindowContent);

        // store new object name as last object name for next checking
        // notice use the full content not TextToRead
        this->LastWindowContent = NewWindowContent;
    }

    //////////////////////////////////////////////////////////////////////////////////////////////////////
    // at this point this->LastWindowContent and this->LastWindowContentSize have been updated if required
    //////////////////////////////////////////////////////////////////////////////////////////////////////
    if (this->LastWindowContent==NULL)
    {
#ifdef _DEBUG
        if (::IsDebuggerPresent())
            ::DebugBreak();
#endif

        goto CleanUp;
    }

    switch(this->ReadStep)
    {
    default:
    case ReadStep_NoContentChangeAllSentencesRead:
    case ReadStep_ContentModifiedAllChangedSentencesRead:
        // nothing to do
        bSuccess = TRUE;
        goto CleanUp;
        // break;

    case ReadStep_NoContentChangeReadNextSentence:
        // go after current read text
        this->LastReadStart+=this->LastReadSize;

        if (*this->LastReadStart==0)
        {
            this->ReadStep = ReadStep_NoContentChangeAllSentencesRead;
            bSuccess = TRUE;
            goto CleanUp;
        }

        // query next sentence read
        this->UpdateReadBufferToReadEndOfSentence();
        break;

    case ReadStep_ContentModifiedReadLetter:
        // go after current read text
        this->LastReadStart+=this->LastReadSize;
        // query letter read
        this->LastReadSize = 1;

        this->ReadStep = ReadStep_ContentModifiedReadWord;
        break;

    case ReadStep_ContentModifiedReadWord:
        // keep LastReadStart unchanged (we want to read the currently modified word)
        // this->LastReadStart+=this->LastReadSize;

        // query next word read
        this->UpdateReadBufferToReadCurrentOrPreviousWord();
        this->ReadStep = ReadStep_ContentModifiedReadBlock;
        break;


    case ReadStep_ContentModifiedReadBlock:
        // keep LastReadStart unchanged (we want to read the currently modified sentence)
        // this->LastReadStart+=this->LastReadSize;

        // query next sentence read
        this->UpdateReadBufferToReadCurrentOrPreviousBlock();

        this->ReadStep = ReadStep_ContentModifiedReadSentence;
        break;

    case ReadStep_ContentModifiedReadSentence:
        // keep LastReadStart unchanged (we want to read the currently modified sentence)
        // this->LastReadStart+=this->LastReadSize;

        // query next sentence read
        this->UpdateReadBufferToReadCurrentOrPreviousSentence();

        this->ReadStep = ReadStep_ContentModifiedReadNextChangedSentence;
        break;

    case ReadStep_ContentModifiedReadNextChangedSentence:
        // go after current read text
        this->LastReadStart+=this->LastReadSize;

        if (   ( *this->LastReadStart == 0 )
            || ( this->LastReadStart > (this->LastChangeStart+this->LastChangeSize ) )
            )
        {
            this->ReadStep = ReadStep_ContentModifiedAllChangedSentencesRead;
            bSuccess = TRUE;
            goto CleanUp;
        }

        // query next sentence read
        this->UpdateReadBufferToReadEndOfSentence();

        break;
    }

    if (this->LastReadSize == 0)
    {
        bSuccess = TRUE;
        goto CleanUp;
    }

    // copy extract of WindowContent into a NULL ended string
    TextToRead = (WCHAR*)malloc((this->LastReadSize+1)*sizeof(WCHAR));
    memcpy(TextToRead,this->LastReadStart,this->LastReadSize*sizeof(WCHAR));
    TextToRead[this->LastReadSize]=0;

    psz = NULL;
    if ( ObjectTextRole // if ObjectTextRole is defined
            && (this->pOptions->bReadRoleSystem) // and if option is set to read role text
            && bFromMouse
            && bPurgeBeforeRead
            && (!bSameWindowHandleAsLastRead)
        )
    {
        SIZE_T Size = wcslen(ObjectTextRole)+1+wcslen(TextToRead)+1; // first +1 for space, second for \0
        psz = (WCHAR*)malloc(Size*sizeof(WCHAR)); 
        _snwprintf(psz,Size,_T("%s %s"),ObjectTextRole,TextToRead);
        free(TextToRead);
        TextToRead = psz;
    }

    // if there's something to read
    if (*TextToRead /* !=0 */)
    {
        if (this->LastTextToRead)
        {
            // second filtering level. 
            // According to what is done previously, if user hits "...", last word will be read for each point (3 repetition) 
            // The same thing occurs on '.' followed by return key
            // so before reading, assume the data to read is not the same as the last one

            // we can't use wcscmp as end of text can only differ by ending non alpha num chars that won't be read
            // if (wcscmp(TextToRead,this->LastTextToRead) == 0)
            if (IsTextToReadTheSame(TextToRead,this->LastTextToRead) )
            {
                bSuccess = TRUE;
                goto CleanUp;
            }
            // else
            free(this->LastTextToRead);
        }


#ifdef _DEBUG
        OutputDebugStringW(TextToRead);
        OutputDebugStringW(L"\r\n");
#endif
        // read text
        this->pVoice->Read(TextToRead,bPurgeBeforeRead,FALSE);

        this->LastTextToRead = wcsdup(TextToRead);
    }

    bSuccess = TRUE;
CleanUp:
    if (ObjectTextRole)
        free(ObjectTextRole);
    if (ObjectTextValue)
        free(ObjectTextValue);
    if (TextToRead)
        free(TextToRead);
    return bSuccess;
}

void CScreenReader::OnClipBoardChangeCallBack(PVOID UserParam)
{
    ((CScreenReader*)UserParam)->OnClipBoardChangeCallBack();
}

void CScreenReader::OnClipBoardChangeCallBack()
{
    if (this->pOptions->bAutoReadClipBoardContent)
        this->ReadClipBoard();
}

//-----------------------------------------------------------------------------
// Name: GetAssociatedObject
// Object: Get object associated to window handle
// Parameters :
//     in  : HWND hWndDialog : handle of the window
//     out :
//     return : associated object if found, NULL if not found
//-----------------------------------------------------------------------------
CScreenReader* CScreenReader::GetAssociatedObject(HWND hWndDialog)
{
    return (CScreenReader*)GetWindowLongPtr(hWndDialog,GWLP_USERDATA);
}

void CScreenReader::HotKeyToVkeyAndModifier(IN DWORD HotKey,OUT BYTE* pVkeyCode,OUT BYTE* pfsModifier)
{
    // The LOBYTE of the LOWORD is the virtual key code of the hot key. The HIBYTE of the LOWORD is the key modifier that specifies the keys that define a hot key combination. The modifier flags can be a combination of the following values.
    //    Value	Meaning
    //    HOTKEYF_ALT	ALT key
    //    HOTKEYF_CONTROL	CONTROL key
    //    HOTKEYF_EXT	Extended key
    //    HOTKEYF_SHIFT	SHIFT key

    WORD lWord = LOWORD(HotKey);
    BYTE VkeyCode = LOBYTE(lWord);
    BYTE HKeyModifier = HIBYTE(lWord);
    // as always ms is not simple and don't use same values for same meanings
    //#define MOD_ALT                         0x0001
    //#define MOD_CONTROL                     0x0002
    //#define MOD_SHIFT                       0x0004
    //
    //#define HOTKEYF_SHIFT           0x01
    //#define HOTKEYF_CONTROL         0x02
    //#define HOTKEYF_ALT             0x04
    BYTE fsModifier=0;
    if (HKeyModifier & HOTKEYF_SHIFT)
        fsModifier|=MOD_SHIFT;
    if (HKeyModifier & HOTKEYF_CONTROL)
        fsModifier|=MOD_CONTROL;
    if (HKeyModifier & HOTKEYF_ALT)
        fsModifier|=MOD_ALT;

    *pVkeyCode = VkeyCode;
    *pfsModifier = fsModifier;
}

void CScreenReader::UpdateHotKey()
{
    if (this->bHotKeyRegistered)
        this->UnregisterHotKeys();

    if (this->pOptions->bUseHotKeys)
        this->RegisterHotKeys();
}
void CScreenReader::RegisterHotKeys()
{
    BYTE VkeyCode;
    BYTE fsModifier;
    BOOL bResult;
    this->HotKeyToVkeyAndModifier(this->pOptions->ReadClipboardHotKey,&VkeyCode,&fsModifier);
    this->bHotKeyRegistered = ::RegisterHotKey(this->hWndDialog,CScreenReader_READ_CLIPBOARD_HOTKEY_ID,(UINT)fsModifier,(UINT)VkeyCode);
    this->HotKeyToVkeyAndModifier(this->pOptions->StartStopHotKey,&VkeyCode,&fsModifier);
    // in case of first call success, due to optimization, RegisterHotKey is not called using next line
    // this->bHotKeyRegistered = this->bHotKeyRegistered || ::RegisterHotKey(this->hWndDialog,CScreenReader_START_STOP_HOTKEY_ID,(UINT)fsModifier,(UINT)VkeyCode);
    bResult = ::RegisterHotKey(this->hWndDialog,CScreenReader_START_STOP_HOTKEY_ID,(UINT)fsModifier,(UINT)VkeyCode);
    this->bHotKeyRegistered = this->bHotKeyRegistered || bResult;

    this->HotKeyToVkeyAndModifier(this->pOptions->SwitchMouseWindowModeHotKey,&VkeyCode,&fsModifier);
    bResult = ::RegisterHotKey(this->hWndDialog,CScreenReader_MOUSE_TO_WINDOW_MODE_HOTKEY_ID,(UINT)fsModifier,(UINT)VkeyCode);
    this->bHotKeyRegistered = this->bHotKeyRegistered || bResult;
}
void CScreenReader::UnregisterHotKeys()
{
    ::UnregisterHotKey(this->hWndDialog,CScreenReader_READ_CLIPBOARD_HOTKEY_ID);
    ::UnregisterHotKey(this->hWndDialog,CScreenReader_START_STOP_HOTKEY_ID);
    ::UnregisterHotKey(this->hWndDialog,CScreenReader_MOUSE_TO_WINDOW_MODE_HOTKEY_ID);
}
void CScreenReader::OnInit()
{
    ::SetTimer(this->hWndDialog,0,this->pOptions->PoolingMouseTime,CScreenReader::TimerProc);
    if (this->pOptions->bUseHotKeys)
        this->RegisterHotKeys();
    
    // start spying clipboard
    this->ClipBoardViewer.SetClipBoardChangeCallBack(CScreenReader::OnClipBoardChangeCallBack,this);
    this->ClipBoardViewer.Register(this->hWndDialog);
}
void CScreenReader::OnClose()
{
    ::KillTimer(this->hWndDialog,0);
    if (this->bHotKeyRegistered)
        this->UnregisterHotKeys();

    // stop spying clipboard
    this->ClipBoardViewer.Unregister();

    ::DestroyWindow(this->hWndDialog);
    this->hWndDialog=NULL;
    ::PostQuitMessage(0);
}

LRESULT CALLBACK CScreenReader::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
    case WM_CREATE:
        {
            CREATESTRUCT* pCreateStruct;
            CScreenReader* pDialogObject; 
            pCreateStruct=(CREATESTRUCT*)lParam;
            pDialogObject=(CScreenReader*)pCreateStruct->lpCreateParams;
            SetWindowLongPtr(hWnd,GWLP_USERDATA,(LONG_PTR)pDialogObject);
            pDialogObject->hWndDialog=hWnd;
            pDialogObject->OnInit();
        }
        break;
    case WM_QUERYENDSESSION:
    case WM_ENDSESSION:
        {
            CScreenReader* pDialogObject; 
            pDialogObject=CScreenReader::GetAssociatedObject(hWnd);
            if (!pDialogObject)
                break;

            if (!pDialogObject->EndSessionProcessed)
            {
                if (pDialogObject->pShutdownBlockReasonCreate)
                    pDialogObject->pShutdownBlockReasonCreate(pDialogObject->hWndDialog,L"Saving Configuration");
                pDialogObject->EndSessionProcessed = TRUE;


                pDialogObject->pOptions->Save();

                if (pDialogObject->pShutdownBlockReasonDestroy)
                    pDialogObject->pShutdownBlockReasonDestroy(pDialogObject->hWndDialog);

                pDialogObject->OnClose();
            }

            if (uMsg == WM_QUERYENDSESSION)
            {
                // WM_QUERYENDSESSION : If an application can terminate conveniently, it should return TRUE; otherwise, it should return FALSE
                return TRUE;
            }
        }
    case WM_QUIT:
    case WM_CLOSE:
        {
            CScreenReader* pDialogObject; 
            pDialogObject=CScreenReader::GetAssociatedObject(hWnd);
            if (!pDialogObject)
                break;

            if (!pDialogObject->EndSessionProcessed)
            {
                // save options
                pDialogObject->pOptions->Save();

                pDialogObject->OnClose();
            }
        }
        break;
    case WM_HOTKEY:
        {
            CScreenReader* pDialogObject; 
            pDialogObject=CScreenReader::GetAssociatedObject(hWnd);
            if (!pDialogObject)
                break;
            if ( pDialogObject->bHotKeyRegistered && (wParam == CScreenReader_READ_CLIPBOARD_HOTKEY_ID) )
            {
                pDialogObject->ReadClipBoard();
            }
            else if ( pDialogObject->bHotKeyRegistered && (wParam == CScreenReader_START_STOP_HOTKEY_ID) )
            {
                pDialogObject->StartStopReading(!pDialogObject->pOptions->bEnabled);
            }
            else if ( pDialogObject->bHotKeyRegistered && (wParam == CScreenReader_MOUSE_TO_WINDOW_MODE_HOTKEY_ID) )
            {
                pDialogObject->SwitchMouseWindowMode();
            }
        }
        break;

    case WM_CHANGECBCHAIN:
    case WM_DRAWCLIPBOARD:
        {
            CScreenReader* pDialogObject; 
            pDialogObject=CScreenReader::GetAssociatedObject(hWnd);
            if (!pDialogObject)
                break;
            pDialogObject->ClipBoardViewer.OnMessage(hWnd, uMsg, wParam, lParam);
        }
        break;

    default:
        return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
    }
    return 0;
}

BOOL CScreenReader::Exit()
{
    if (!this->hWndDialog)
        return FALSE;

    // use post message to allow current message to be fully processed before closing
    ::PostMessage(this->hWndDialog,WM_CLOSE,0,0);

    return TRUE;
}

BOOL CScreenReader::WaitApplicationEnd()
{
    WNDCLASSEX wc;
    HWND hwnd;
    MSG Msg;

    // if CScreenReader_CLASS_NAME is not already registered
    if (!GetClassInfoEx(this->hInstance, CScreenReader_CLASS_NAME , &wc))
    {
        // register it

        wc.cbSize        = sizeof(WNDCLASSEX);
        wc.style         = 0;
        wc.lpfnWndProc   = CScreenReader::WndProc;
        wc.cbClsExtra    = 0;
        wc.cbWndExtra    = 0;
        wc.hInstance     = this->hInstance;
        wc.hIcon         = NULL;
        wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
        wc.hbrBackground = (HBRUSH)COLOR_WINDOWFRAME;
        wc.lpszMenuName  = NULL;
        wc.lpszClassName = CScreenReader_CLASS_NAME;
        wc.hIconSm       = NULL;
        if(!RegisterClassEx(&wc))
        {
            ::MessageBox(NULL, _T("Window Registration Failed"), _T("Error"),MB_OK|MB_ICONERROR);
            return FALSE;
        }
    }

    // create the dialog window
    hwnd = ::CreateWindowEx(0,
                            CScreenReader_CLASS_NAME,
                            _T(""),
                            WS_DLGFRAME,
                            0,
                            0, 
                            0, 
                            0,
                            HWND_DESKTOP,
                            NULL,
                            this->hInstance,
                            this);

    if(hwnd == NULL)
    {
        ::MessageBox(NULL, _T("Window Creation Failed"), _T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

    BOOL bRet;
    while( (bRet = ::GetMessage( &Msg, NULL, 0, 0 )) != 0)
    { 
        if (bRet == -1)
        {
            // handle the error and possibly exit
        }
        else
        {
            ::TranslateMessage(&Msg); 
            ::DispatchMessage(&Msg); 
        }
    }
    // return (DWORD)Msg.wParam;
    return TRUE;
}