#include "voice.h"



CVoice::CVoice(void)
{
    this->pVoice = NULL;
}

CVoice::~CVoice(void)
{
    this->UnLoad();
}

BOOL CVoice::Load()
{
    // unload previous voice if any
    this->UnLoad();

    // create COM object
    HRESULT hResult = ::CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpVoice, (void **)&pVoice);
    if( FAILED( hResult ) || (pVoice == NULL) )
        return FALSE;

    return TRUE;
}
#define CVoice_EnumTokens_NAME_PREFIX_ATTRIBUTE L"Name="
BOOL CVoice::GetVoiceToken(WCHAR* TTSName, OUT ISpObjectToken** ppToken)
{
    IEnumSpObjectTokens* pEnum;
    BOOL bSuccess;
    HRESULT hResult;
    WCHAR* ReqAttribs = NULL;
    *ppToken = NULL;
    
    bSuccess = FALSE;
    ReqAttribs = new WCHAR[wcslen(TTSName)+wcslen(CVoice_EnumTokens_NAME_PREFIX_ATTRIBUTE)+1];
    wcscpy(ReqAttribs,CVoice_EnumTokens_NAME_PREFIX_ATTRIBUTE);
    wcscat(ReqAttribs,TTSName);
    
    hResult = ::SpEnumTokens(SPCAT_VOICES, ReqAttribs, NULL, &pEnum);
    if( FAILED(hResult) || (pEnum==0) )
        goto CleanUp;

    //Get the closest token
    hResult = pEnum->Next(1, ppToken, NULL);
        
    bSuccess = SUCCEEDED(hResult) && (*ppToken != 0);

CleanUp:
    if (pEnum)
        pEnum->Release();
    if (ReqAttribs)
        delete ReqAttribs;

    return bSuccess;
}

BOOL CVoice::SetVoice(WCHAR* TTSName)
{
    if (!this->IsLoaded())
        return FALSE;

    ISpObjectToken* pToken;
    if (!this->GetVoiceToken(TTSName,&pToken))
        return FALSE;

    this->StopReading();
    HRESULT hResult = this->pVoice->SetVoice(pToken);
	this->pVoice->SetVolume(100);
	this->pVoice->SetRate(0);
    pToken->Release();

    return ( SUCCEEDED( hResult ) );
}

// Speed -10;+10
// Volume 0;100
BOOL CVoice::SetSpeedAndVolume(LONG Speed,USHORT Volume)
{
    if (!this->IsLoaded())
        return FALSE;
    HRESULT hResult;
    HRESULT hResult2;
    hResult=this->pVoice->SetVolume(Volume);
    hResult2=this->pVoice->SetRate(Speed);

    return ( SUCCEEDED( hResult ) && SUCCEEDED( hResult2 ) );
}

BOOL CVoice::GetSpeedAndVolume(OUT LONG* pSpeed,OUT USHORT* pVolume)
{
    if (!this->IsLoaded())
        return FALSE;
    HRESULT hResult;
    HRESULT hResult2;
    hResult=this->pVoice->GetVolume(pVolume);
    hResult2=this->pVoice->GetRate(pSpeed);

    return ( SUCCEEDED( hResult ) && SUCCEEDED( hResult2 ) );
}

BOOL CVoice::UnLoad()
{
    if (!this->IsLoaded())
        return TRUE;

    this->StopReading();
    this->WaitEndOfReading();

    // assume this->pVoice is NULL before releasing
    ISpVoice* pTmpVoice;
    pTmpVoice = this->pVoice;
    this->pVoice = NULL;
    pTmpVoice->Release();
    
    return TRUE;
}
BOOL CVoice::IsLoaded()
{
    return (this->pVoice != NULL);
}
BOOL CVoice::Read(WCHAR* TextToRead,BOOL bPurgeBeforeRead,BOOL bSynchronous)
{
    if (!this->IsLoaded())
        return FALSE;

	HRESULT hResult;
    DWORD dwFlags = 0;
    if (!bSynchronous)
        dwFlags |= SPF_ASYNC;
    if (bPurgeBeforeRead)
        dwFlags |= SPF_PURGEBEFORESPEAK;
	hResult = this->pVoice->Speak(TextToRead, dwFlags, NULL);
    return SUCCEEDED( hResult );
}

BOOL CVoice::StopReading()
{
    if (!this->IsLoaded())
        return FALSE;

	HRESULT hResult;
	hResult = this->pVoice->Speak(NULL, SPF_ASYNC | SPF_PURGEBEFORESPEAK, NULL);
    return SUCCEEDED( hResult );
}

BOOL CVoice::WaitEndOfReading(ULONG MsTimeOut)
{
    if (!this->IsLoaded())
        return FALSE;

	HRESULT hResult;
	hResult = this->pVoice->WaitUntilDone(MsTimeOut);
    // return SUCCEEDED( hResult ); Don't use this ! WaitUntilDone return S_FALSE and m$ SUCCEEDED(S_FALSE) == TRUE
    return (hResult==S_OK);
}

BOOL CVoice::IsReading()
{
    return !this->WaitEndOfReading(0);
}

HANDLE CVoice::GetSpeakCompleteEvent()
{
    if (!this->IsLoaded())
        return NULL;
    return this->pVoice->SpeakCompleteEvent();
}

BOOL CVoice::EnumVoices(tagCallBackEnumVoices CallBack,PVOID UserParam)
{
    //Pointer to token enumerator
    IEnumSpObjectTokens* pEnum;
    HRESULT hResult;
    WCHAR* pszDesciptionString;
    ISpObjectToken* pToken;
    LANGID LangId;
    BOOL bContinueEnum;
	WCHAR* CultureSplitter;

    if (::IsBadCodePtr((FARPROC)CallBack))
        return FALSE;

    // Get a token enumerator for tts voices available
    hResult = ::SpEnumTokens(SPCAT_VOICES, NULL, NULL, &pEnum);
    if ( FAILED(hResult) || (pEnum == 0) )
        return FALSE;

    pToken = NULL;
    LangId = ::SpGetUserDefaultUILanguage();
    bContinueEnum = TRUE;
    while ( SUCCEEDED( pEnum->Next(1, &pToken, NULL) ) )
    {
        if (pToken == NULL)
            break;

        // Get a string which describes the token, in our case, the voice name 
        pszDesciptionString = NULL;
        hResult = ::SpGetDescription( pToken, &pszDesciptionString, LangId );
        if ( SUCCEEDED(hResult) && pszDesciptionString)
        {
			//MSDN : "In previous versions of Windows, Control Panel identifies TTS voices by name; 
			//for example, Microsoft Sam. In the Vista operating system, Control Panel identifies TTS voices by name and culture; 
			//for example, Microsoft Anna  English (United States). 
			//In this case, Microsoft Anna is the name of the voice and English (United States) is its culture. 
			//The name parameter passed to the SelectVoice method should specify only the name of the voice and not its culture." <---
			// --> So once again MS is happy to make shit developers (Why do they get so much lamers incompatible ideas ???)
			CultureSplitter = wcsstr(pszDesciptionString,L" - ");
			if (CultureSplitter)
			{
				*CultureSplitter=0;
			}

            bContinueEnum = CallBack(pszDesciptionString,UserParam);
            ::CoTaskMemFree(pszDesciptionString);
        }

        // Release the token itself  
        pToken->Release();  
        pToken = NULL;

        if (!bContinueEnum)
            break;
    }

    pEnum->Release();

    return TRUE;
}


BOOL CVoice::DisplayUI(HWND hwndParent)
{
    if (!this->IsLoaded())
        return NULL;

    HRESULT hResult;
    BOOL bSupported;

    bSupported = FALSE;
    hResult = this->pVoice->IsUISupported(SPDUI_EngineProperties, NULL, NULL, &bSupported);
    if (!bSupported)
        return FALSE;
    // Display properties UI for the current TTS engine.
	hResult = this->pVoice->DisplayUI(hwndParent, L"Text-to-speech engine's properties", SPDUI_EngineProperties, NULL, NULL);
    return SUCCEEDED( hResult );
}

/*
// init
CComPtr<ISpVoice> cpVoice;
hResult = cpVoice.CoCreateInstance(CLSID_SpVoice);
hResult = cpVoice->Speak(L" Hello, world ", 0, NULL);
// or
CComPtr<IspVoice> g_cpVoice;
hResult = g_cpRecoCtxt->GetVoice(&g_cpVoice);
*/

/*
// Get the token representing the current voice
HRESULT hResult = g_cpVoice->GetVoice( &pToken );

if ( SUCCEEDED( hResult ) )
{    
    // Get the current token ID, and compare it against others to figure out
    // which desciption string is the one currently selected.   
    hResult = pToken->GetId( &pszCurTokenId);   
    if ( SUCCEEDED( hResult ) )    
    {
        ulIndex = 0;
        while ( (ulIndex < ulNumTokens) && (0 != _wcsicmp(pszCurTokenId, ppszTokenIds[ulIndex]) )
        { 
            ulIndex++;         
        }

        // We found it, so set the current index to that of the current token        
        if ( ulIndex < ulNumTokens )           
        {
            ulCurToken = ulIndex;
        }

        CoTaskMemFree( pszCurTokenId);    
    }
    pToken->Release();
}
*/