#include "VoiceOptions.h"
#include "resource.h"
#include <CommCtrl.h>

CVoiceOptions::CVoiceOptions(CVoice* pVoice,COptions* pOptions)
{
    this->pVoice = pVoice;
    this->pOptions = pOptions;
    this->bApplyChanges = FALSE;
}


CVoiceOptions::~CVoiceOptions(void)
{
}

INT_PTR CVoiceOptions::Show(HINSTANCE hInstance,HWND hWndDialog,CVoice* pVoice,COptions* pOptions)
{
    INT_PTR Ret;
    CVoiceOptions* pDialog=new CVoiceOptions(pVoice,pOptions);

    Ret=pDialog->CDialogBase::Show(hInstance,hWndDialog,IDD_DIALOG_VOICE_CONFIG,IDI_ICON_VOICES_OPTIONS);
    delete pDialog;
    return Ret;
}

void CVoiceOptions::OnInit()
{
    this->SetDlgItemText(IDC_EDIT_VOICE_CONFIG_TEST,_T("This is a test"));
    // original speed and volume
    this->pVoice->GetSpeedAndVolume(&this->OriginalRate,&this->OriginalVolume);
    this->CurrentRate=this->OriginalRate;
    this->CurrentVolume=this->OriginalVolume;

    this->SendDlgMessage(IDC_SLIDER_VOICE_SPEED,TBM_SETRANGE,TRUE,MAKELONG(0,20));// MAKELONG(-10,10)
    this->SendDlgMessage(IDC_SLIDER_VOICE_VOLUME,TBM_SETRANGE,TRUE,MAKELONG(0,100));

    this->SendDlgMessage(IDC_SLIDER_VOICE_SPEED,TBM_SETPOS,TRUE,this->CurrentRate+10);
    this->SendDlgMessage(IDC_SLIDER_VOICE_VOLUME,TBM_SETPOS,TRUE,this->CurrentVolume);

    this->SliderSpeed=this->GetDlgItem(IDC_SLIDER_VOICE_SPEED);
    this->SliderVolume=this->GetDlgItem(IDC_SLIDER_VOICE_VOLUME);
}
BOOL CVoiceOptions::OnClose()
{
    if (this->bApplyChanges)
    {
        // save vol and speed (vol and speed are already set)
        this->pOptions->VoiceSpeed = this->CurrentRate;
        this->pOptions->VoiceVolume = this->CurrentVolume;
        this->pOptions->SaveVoiceSpeedAndVolume();
    }
    else
    {
        // restore original speed + volume
        this->pVoice->SetSpeedAndVolume(this->OriginalRate,this->OriginalVolume);
    }

    return TRUE;
}

void CVoiceOptions::OnCancel()
{
    this->Close();
}
void CVoiceOptions::OnOk()
{
    this->bApplyChanges = TRUE;
    // /!\ warning : OnClose is always called
    this->Close();
}

void CVoiceOptions::OnCommand(WPARAM wParam,LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);
    switch (LOWORD(wParam))
    {
    case IDOK:
        this->OnOk();
        break;
    case IDCANCEL:
        this->OnCancel();
        break;
    case IDC_BUTTON_VOICE_CONFIG_TEST:
        this->OnTest();
        break;
    }
}

void CVoiceOptions::OnTest()
{
    TCHAR Txt[1024];
    this->GetDlgItemText(IDC_EDIT_VOICE_CONFIG_TEST,Txt,_countof(Txt));
    this->pVoice->Read(Txt,TRUE,FALSE);
}
LRESULT CVoiceOptions::OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam)
{
    DWORD dwPos;    // current position of slider 
    switch (uMsg)
    {
    case WM_HSCROLL:
        switch (LOWORD(wParam))
        {
        case TB_ENDTRACK:
            if ((HWND)lParam == this->SliderSpeed)
            {
                dwPos = (DWORD)SendMessage(this->SliderSpeed, TBM_GETPOS, 0, 0);
                this->CurrentRate = dwPos-10;
                this->pVoice->SetSpeedAndVolume(this->CurrentRate,this->CurrentVolume);
            }
            else if ((HWND)lParam == this->SliderVolume)
            {
                dwPos = (DWORD)SendMessage(this->SliderVolume, TBM_GETPOS, 0, 0);
                this->CurrentVolume =(USHORT) dwPos;
                this->pVoice->SetSpeedAndVolume(this->CurrentRate,this->CurrentVolume);
            }
            break;
        }
        break;
    }
    return FALSE;
}
