#pragma once

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0501 // for xp os
#endif

#include "../Tools/gui/NotifyIcon/NotifyIcon.h"
#include "../Tools/gui/Menu/PopUpMenu.h"

#include <windows.h>
#include "resource.h"
#include "ObjectNameFromMouse.h"
#include "Options.h"
#include "Voice.h"
#include "about.h"
#include "ReadUI.h"


#define CScreenReader_CLASS_NAME                        _T("CScreenReaderClassName")
#define CScreenReader_VOICE_NAME_MAX_LEN                COption_VOICE_NAME_MAX_LEN

class CScreenReader
{
protected:
    HINSTANCE hInstance;
    HWND hWndDialog;
    CVoice* pVoice;
    COptions* pOptions;
    CNotifyIcon* pNotifyIcon;
    WCHAR* LastObjectName;
    CPopUpMenu* pPopUpMenu;
    CPopUpMenu* pPopUpMenuVoices;
    HANDLE hEvtExit;

    UINT MenuIdRead;
    UINT MenuIdReadClipboard;
    UINT MenuIdEnabled;
    UINT MenuIdReadRole;
    UINT MenuIdAbout;
    UINT MenuIdExit;
    UINT CurrentVoiceMenuId;


    BOOL FreeMemory();
    static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    void OnClose();
    void OnInit();
    static CScreenReader* GetAssociatedObject(HWND hWndDialog);
    BOOL WaitApplicationEnd();
    BOOL Exit();
    BOOL ReadAccessibleObjectFromMouse();
    static DWORD WINAPI ReadingThread( LPVOID lpParam );
    static BOOL CallBackEnumVoices(WCHAR* TTSName,PVOID UserParam);
    static BOOL NotifyIconCallBack(LPPOINT pCursorPos,PVOID UserParam);
public:
    CScreenReader(HINSTANCE hInstance);
    ~CScreenReader();
    
    BOOL Run();
};

int WINAPI WinMain (HINSTANCE hInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpCmdLine,
                    int nCmdShow
                    )
{
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);
    UNREFERENCED_PARAMETER(nCmdShow);

    CScreenReader ScreenReader(hInstance);
    if (!ScreenReader.Run())
        return -1;

    return 0;
}

BOOL CScreenReader::CallBackEnumVoices(WCHAR* TTSName,PVOID UserParam)
{
    CPopUpMenu* pPopUpMenuVoices = (CPopUpMenu*)UserParam;
#ifdef _DEBUG
    OutputDebugStringW(TTSName);
    OutputDebugStringW(L"\r\n");
#endif

    // add voice to popupmenu
    pPopUpMenuVoices->Add(TTSName);

    return TRUE;
}

BOOL CScreenReader::NotifyIconCallBack(LPPOINT pCursorPos,PVOID UserParam)
{
    UINT ClickedMenuId;
    CScreenReader* pCScreenReader = (CScreenReader*)UserParam;
    ClickedMenuId = pCScreenReader->pPopUpMenu->Show(pCursorPos->x,pCursorPos->y,pCScreenReader->pNotifyIcon->GetAssocatedNotifyIconHwnd(),FALSE,TRUE);
    if (ClickedMenuId)
    {
        if (ClickedMenuId == pCScreenReader->MenuIdRead)
        {
            CReadUI::Show(pCScreenReader->hInstance,pCScreenReader->hWndDialog,pCScreenReader->pVoice);
        }
        else if (ClickedMenuId == pCScreenReader->MenuIdReadClipboard)
        {
            ::OpenClipboard(pCScreenReader->hWndDialog);
            WCHAR* pw = (WCHAR*)::GetClipboardData(CF_UNICODETEXT);
            if (pw)
            {
                pCScreenReader->pVoice->Read(pw,TRUE);
            }
#ifdef _DEBUG
            else
                ::MessageBox(NULL, _T("Empty Cliboard"), _T("Error"),MB_OK|MB_ICONERROR);
#endif
            ::CloseClipboard();
        }
        else if (ClickedMenuId == pCScreenReader->MenuIdEnabled)
        {
            BOOL bOldState = pCScreenReader->pPopUpMenu->IsChecked(pCScreenReader->MenuIdEnabled);
            pCScreenReader->pOptions->bEnabled = !bOldState;
            pCScreenReader->pPopUpMenu->SetCheckedState(pCScreenReader->MenuIdEnabled,pCScreenReader->pOptions->bEnabled);

            if (pCScreenReader->pOptions->bEnabled)
            {
                pCScreenReader->pNotifyIcon->SetIcon(pCScreenReader->hInstance,IDI_ICON_SCREEN_READER);
            }
            else
            {
                pCScreenReader->pNotifyIcon->SetIcon(pCScreenReader->hInstance,IDI_ICON_SCREEN_READER_DISABLED);
            }
        }
        else if (ClickedMenuId == pCScreenReader->MenuIdReadRole)
        {
            BOOL bOldState = pCScreenReader->pPopUpMenu->IsChecked(pCScreenReader->MenuIdReadRole);
            pCScreenReader->pOptions->bReadRoleSystem = !bOldState;
            pCScreenReader->pPopUpMenu->SetCheckedState(pCScreenReader->MenuIdReadRole,pCScreenReader->pOptions->bReadRoleSystem);
        }
        else if (ClickedMenuId == pCScreenReader->MenuIdAbout)
        {
            CAbout::Show(pCScreenReader->hInstance,pCScreenReader->hWndDialog,NULL);
        }
        else if (ClickedMenuId == pCScreenReader->MenuIdExit)
        {
            pCScreenReader->Exit();
        }
        else
        {
            // voice name
            WCHAR VoiceName[CScreenReader_VOICE_NAME_MAX_LEN];
            pCScreenReader->pPopUpMenu->GetText(ClickedMenuId,VoiceName,CScreenReader_VOICE_NAME_MAX_LEN);
            if (pCScreenReader->pVoice->SetVoice(VoiceName))
            {
                // store name into options
                wcscpy(pCScreenReader->pOptions->VoiceName,VoiceName);

                // unselect last voice menu item
                if (pCScreenReader->CurrentVoiceMenuId)
                    pCScreenReader->pPopUpMenu->SetCheckedState(pCScreenReader->CurrentVoiceMenuId,FALSE);

                // select new voice menu item
                pCScreenReader->pPopUpMenu->SetCheckedState(ClickedMenuId,TRUE);

                // store new voice menu item
                pCScreenReader->CurrentVoiceMenuId = ClickedMenuId;
            }
        }
    }

    // return TRUE to restore focus to notify icon, FALSE to keep focus
    return FALSE;
}

DWORD WINAPI CScreenReader::ReadingThread( LPVOID lpParam )
{
    ::CoInitialize(0);
    CScreenReader* pCScreenReader = (CScreenReader*)lpParam;
    DWORD dwRet;
    for(;;)
    {
        // do pooling on exit event
        dwRet = ::WaitForSingleObject(pCScreenReader->hEvtExit,pCScreenReader->pOptions->PoolingTime );
        if (dwRet == WAIT_OBJECT_0)
        {
            // exit event has occurred -> end thread
            ::CoUninitialize();
            return 0;
        }

        // if read option is enabled 
        if (pCScreenReader->pOptions->bEnabled)
            pCScreenReader->ReadAccessibleObjectFromMouse();
    }
}

CScreenReader::CScreenReader(HINSTANCE hInstance)
{
    this->hInstance = hInstance;
    this->pVoice = NULL;
    this->LastObjectName = NULL;
    this->pOptions = NULL;
    this->pNotifyIcon = NULL;
    this->hWndDialog = NULL;
    this->pPopUpMenu = NULL;
    this->pPopUpMenuVoices = NULL;
    this->MenuIdRead = 0;
    this->MenuIdReadClipboard = 0;
    this->MenuIdEnabled = 0;
    this->MenuIdReadRole = 0;
    this->MenuIdAbout = 0;
    this->MenuIdExit = 0;
    this->CurrentVoiceMenuId = 0;
}
CScreenReader::~CScreenReader()
{
    this->FreeMemory();
}

BOOL CScreenReader::Run()
{
    if (FAILED(::CoInitialize(NULL)))
        return FALSE;

    this->pVoice = new CVoice();// must be after ::CoInitialize();
    if (!this->pVoice->Load())
    {
        // sapi 5 not found
        MessageBox(NULL,_T("This software require Microsoft Speech API 5\r\n"),_T("Error"),MB_OK|MB_ICONERROR);
        return FALSE;
    }

    // load options
    this->pOptions = new COptions();
    this->pOptions->Load();

    // create notify icon
    UINT IdIcon = IDI_ICON_SCREEN_READER;
    if (!this->pOptions->bEnabled)
        IdIcon = IDI_ICON_SCREEN_READER_DISABLED;
    this->pNotifyIcon = new CNotifyIcon( this->hInstance,IdIcon,_T("Screen Reader") );

    // create popup menu associated to notify icon
    this->pPopUpMenu = new CPopUpMenu();

    // add all known voices to menu
    this->pPopUpMenuVoices = new CPopUpMenu(this->pPopUpMenu);

    this->MenuIdRead = this->pPopUpMenu->Add(_T("Read"),IDI_ICON_READ,this->hInstance);
    this->MenuIdReadClipboard = this->pPopUpMenu->Add(_T("Read ClipBoard"),IDI_ICON_READ_CLIPBOARD,this->hInstance);

    this->pPopUpMenu->AddSeparator();
    this->MenuIdEnabled = this->pPopUpMenu->Add(_T("Enabled"));
    this->MenuIdReadRole = this->pPopUpMenu->Add(_T("Read Role"));
    this->pPopUpMenu->AddSubMenu(_T("Voices"),this->pPopUpMenuVoices,IDI_ICON_VOICES,this->hInstance);
    this->pVoice->EnumVoices(CallBackEnumVoices,this->pPopUpMenuVoices);

    this->pPopUpMenu->AddSeparator();
    this->MenuIdAbout = this->pPopUpMenu->Add(_T("About"),IDI_ICON_ABOUT,this->hInstance);
    this->pPopUpMenu->AddSeparator();
    this->MenuIdExit = this->pPopUpMenu->Add(_T("Exit"),IDI_ICON_EXIT,this->hInstance);

    // adjust popupmenu, notify icon, and TTS engine according to options
    if (this->pOptions->bEnabled)
        this->pPopUpMenu->SetCheckedState(this->MenuIdEnabled,TRUE);

    if (this->pOptions->bReadRoleSystem)
        this->pPopUpMenu->SetCheckedState(this->MenuIdReadRole,TRUE);

    // loop though voice menu to check used voice
    int CurrentMenuId;
    WCHAR VoiceName[CScreenReader_VOICE_NAME_MAX_LEN];
    for (int Cnt = 0; Cnt < this->pPopUpMenuVoices->GetItemCount(); Cnt++)
    {
        CurrentMenuId = this->pPopUpMenuVoices->GetID(Cnt);
        this->pPopUpMenuVoices->GetText(CurrentMenuId,VoiceName,CScreenReader_VOICE_NAME_MAX_LEN);
        // if voice name matchs
        if (wcsicmp(VoiceName,this->pOptions->VoiceName) == 0)
        {
            // check voice name menu
            this->pPopUpMenu->SetCheckedState(CurrentMenuId,TRUE);

            // store selected menu id
            this->CurrentVoiceMenuId = CurrentMenuId;

            // load specified voice
            this->pVoice->SetVoice(VoiceName);

            break;
        }
    }
    

    // set notify icon clicks callbacks
    this->pNotifyIcon->SetLeftClickCallBack(CScreenReader::NotifyIconCallBack,this);
    this->pNotifyIcon->SetRightClickCallBack(CScreenReader::NotifyIconCallBack,this);
    // may todo : change it to start / stop or show read raw text ?
    this->pNotifyIcon->SetDoubleClickCallBack(CScreenReader::NotifyIconCallBack,this);


    // create the exit event
    this->hEvtExit = ::CreateEvent(NULL,FALSE,FALSE,NULL);
    // create reading thread
    HANDLE hReadingThread = ::CreateThread(NULL,0,CScreenReader::ReadingThread,(LPVOID)this,0,NULL);   

    // display notify icon
    this->pNotifyIcon->Show();

    // wait exit message
    this->WaitApplicationEnd();
    
    // ask to close reading thread
    ::SetEvent(hEvtExit);

    if (hReadingThread)
    {
        // wait end of reading thread
        WaitForSingleObject(hReadingThread,INFINITE);
        // close thread handle
        ::CloseHandle(hReadingThread);
    }

    // close event after reading thread end (handle used by thread)
    ::CloseHandle(hEvtExit);

    // destroy object after reading thread end

    delete pNotifyIcon;

    delete this->pPopUpMenuVoices;
    this->pPopUpMenuVoices = NULL;

    delete this->pPopUpMenu;
    this->pPopUpMenu = NULL;

    delete this->pVoice; // must be before ::CoUninitialize();
    this->pVoice=NULL;

    // save options
    this->pOptions->Save();
    delete this->pOptions;
    this->pOptions = NULL;

    ::CoUninitialize();
    return TRUE;
}


BOOL CScreenReader::FreeMemory()
{
    if (this->LastObjectName)
        free(this->LastObjectName);
    return TRUE;
}


BOOL CScreenReader::ReadAccessibleObjectFromMouse()
{
    BOOL bSuccess = FALSE;
    WCHAR* ObjectName=NULL;
    WCHAR* ObjectTextRole=NULL;
    WCHAR* psz = NULL;
    WCHAR* TextToRead = NULL;

    // if voice is not loaded
    if (!this->pVoice->IsLoaded())
        return FALSE;

    // if text was successfully retreived
    if (CObjectNameFromMouse::GetObjectNameFromMouse(&ObjectName,&ObjectTextRole))
    {
        if (!ObjectName)
            goto CleanUp;

        // if the mouse is on the same object as the last time, don't read it (avoid repetition)
        if (LastObjectName)
        {
            if (wcsicmp(LastObjectName , ObjectName) == 0) 
            {
                bSuccess = TRUE;
                goto CleanUp;
            }
            // else
            free(this->LastObjectName);
        }
        // store new object name as last object name for next checking
        LastObjectName = wcsdup(ObjectName);

        if ( ObjectTextRole // if ObjectTextRole is defined
             && (this->pOptions->bReadRoleSystem) // and if option is set to read role text
           )
        {
            psz = new WCHAR[wcslen(ObjectTextRole)+1+wcslen(ObjectName)+1]; // first +1 for space, second for \0
            wcscpy(psz,ObjectTextRole);
            wcscat(psz,L" ");
            wcscat(psz,ObjectName);
            TextToRead = psz;
        }
        else
        {
            TextToRead = ObjectName;
        }

        // if there's something to read
        if (*TextToRead /* !=0 */)
            // read text
            this->pVoice->Read(TextToRead,TRUE);

        // delete may allocated resource
        if (psz)
            delete psz;
    }

    bSuccess = TRUE;
CleanUp:
    if (ObjectName)
        free(ObjectName);
    if (ObjectTextRole)
        free(ObjectTextRole);

    return bSuccess;
}

//-----------------------------------------------------------------------------
// Name: GetAssociatedObject
// Object: Get object associated to window handle
// Parameters :
//     in  : HWND hWndDialog : handle of the window
//     out :
//     return : associated object if found, NULL if not found
//-----------------------------------------------------------------------------
CScreenReader* CScreenReader::GetAssociatedObject(HWND hWndDialog)
{
    return (CScreenReader*)GetWindowLongPtr(hWndDialog,GWLP_USERDATA);
}

void CScreenReader::OnInit()
{

}
void CScreenReader::OnClose()
{
    ::DestroyWindow(this->hWndDialog);
    this->hWndDialog=NULL;
    ::PostQuitMessage(0);
}

LRESULT CALLBACK CScreenReader::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
    case WM_CREATE:
        {
            CREATESTRUCT* pCreateStruct;
            CScreenReader* pDialogObject; 
            pCreateStruct=(CREATESTRUCT*)lParam;
            pDialogObject=(CScreenReader*)pCreateStruct->lpCreateParams;
            SetWindowLongPtr(hWnd,GWLP_USERDATA,(LONG_PTR)pDialogObject);
            pDialogObject->hWndDialog=hWnd;
            pDialogObject->OnInit();
        }
        break;
    case WM_QUERYENDSESSION:
    case WM_QUIT:
    case WM_CLOSE:
        {
            CScreenReader* pDialogObject; 
            pDialogObject=CScreenReader::GetAssociatedObject(hWnd);
            if (!pDialogObject)
                break;
            pDialogObject->OnClose();

            if (uMsg==WM_QUERYENDSESSION)
            {
                // WM_QUERYENDSESSION : If an application can terminate conveniently, it should return TRUE; otherwise, it should return FALSE
                return TRUE;
            }
        }
        break;
    default:
        return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
    }
    return 0;
}

BOOL CScreenReader::Exit()
{
    if (!this->hWndDialog)
        return FALSE;

    // use post message to allow current message to be fully processed before closing
    ::PostMessage(this->hWndDialog,WM_CLOSE,0,0);

    return TRUE;
}

BOOL CScreenReader::WaitApplicationEnd()
{
    WNDCLASSEX wc;
    HWND hwnd;
    MSG Msg;

    // if CScreenReader_CLASS_NAME is not already registered
    if (!GetClassInfoEx(this->hInstance, CScreenReader_CLASS_NAME , &wc))
    {
        // register it

        wc.cbSize        = sizeof(WNDCLASSEX);
        wc.style         = 0;
        wc.lpfnWndProc   = CScreenReader::WndProc;
        wc.cbClsExtra    = 0;
        wc.cbWndExtra    = 0;
        wc.hInstance     = this->hInstance;
        wc.hIcon         = NULL;
        wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
        wc.hbrBackground = (HBRUSH)COLOR_WINDOWFRAME;
        wc.lpszMenuName  = NULL;
        wc.lpszClassName = CScreenReader_CLASS_NAME;
        wc.hIconSm       = NULL;
        if(!RegisterClassEx(&wc))
        {
            ::MessageBox(NULL, _T("Window Registration Failed"), _T("Error"),MB_OK|MB_ICONERROR);
            return FALSE;
        }
    }

    // create the dialog window
    hwnd = ::CreateWindowEx(0,
        CScreenReader_CLASS_NAME,
        _T(""),
        WS_DLGFRAME,
        0,
        0, 
        0, 
        0,
        HWND_DESKTOP,
        NULL,
        this->hInstance,
        this);

    if(hwnd == NULL)
    {
        ::MessageBox(NULL, _T("Window Creation Failed"), _T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

    BOOL bRet;
    while( (bRet = ::GetMessage( &Msg, NULL, 0, 0 )) != 0)
    { 
        if (bRet == -1)
        {
            // handle the error and possibly exit
        }
        else
        {
            ::TranslateMessage(&Msg); 
            ::DispatchMessage(&Msg); 
        }
    }
    // return (DWORD)Msg.wParam;
    return TRUE;
}