#include "ClipboardViewer.h"


CClipboardViewer::CClipboardViewer(void)
{
    this->hwndNextViewer = NULL;
    this->hWndDlg = NULL;
}

CClipboardViewer::~CClipboardViewer(void)
{
    if (this->hwndNextViewer)
        this->Unregister();
}

BOOL CClipboardViewer::SetClipBoardChangeCallBack(pfOnClipBoardChangeCallBack pFunction,PVOID UserParam)
{
    if (pFunction != NULL)// NUUL can be used to remove call back
    {
        if (::IsBadCodePtr( (FARPROC)pFunction ) )
            return FALSE;
    }

    this->OnClipBoardChangeCallBack=pFunction;
    this->OnClipBoardChangeCallBackUserParam=UserParam;
    return TRUE;
}

BOOL CClipboardViewer::Register(HWND hWndDlg, BOOL CallOnClipBoardChangeCallBackForCurrentClipBoardContent)
{
    this->FirstMessageAfterRegistration = TRUE;
    this->CallOnClipBoardChangeCallBackForCurrentClipBoardContent = CallOnClipBoardChangeCallBackForCurrentClipBoardContent;
    this->hWndDlg = hWndDlg;

    // Add the window to the clipboard viewer chain. 
    this->hwndNextViewer = ::SetClipboardViewer(this->hWndDlg);

    return (this->hwndNextViewer!=NULL);// use GetLastError for more informations
}
BOOL CClipboardViewer::Unregister()
{
    if (!this->hwndNextViewer)
        return FALSE;
    // Remove hWndDlg from the clipboard viewer chain
    BOOL bRetValue = ::ChangeClipboardChain(this->hWndDlg, this->hwndNextViewer); 
    this->hwndNextViewer = NULL;
    return bRetValue;
}

LRESULT CClipboardViewer::OnMessage(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{   
    BOOL CallOnClipBoardChangeCallBack;

    switch(uMsg)
    {
        case WM_CHANGECBCHAIN: 
            // If the next window is closing, repair the chain. 
            if ((HWND) wParam == this->hwndNextViewer) 
                this->hwndNextViewer = (HWND) lParam; 
 
            // Otherwise, pass the message to the next link. 
            else if (this->hwndNextViewer != NULL) 
                ::SendMessage(this->hwndNextViewer, uMsg, wParam, lParam); 
            break;


        case WM_DRAWCLIPBOARD:  // clipboard contents changed.
            CallOnClipBoardChangeCallBack = TRUE;
            if (!this->CallOnClipBoardChangeCallBackForCurrentClipBoardContent)
            {
                if (this->FirstMessageAfterRegistration)
                {
                    CallOnClipBoardChangeCallBack = FALSE;
                    this->FirstMessageAfterRegistration = FALSE;
                }
            }

            if (this->OnClipBoardChangeCallBack && CallOnClipBoardChangeCallBack)
                this->OnClipBoardChangeCallBack(this->OnClipBoardChangeCallBackUserParam);
 
            // Pass the message to the next window in clipboard 
            // viewer chain.
            if (this->hwndNextViewer != NULL)  
                ::SendMessage(this->hwndNextViewer, uMsg, wParam, lParam); 
            break; 

        default:
            return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
    }

    return (LRESULT)0;
}