/*
Copyright (C) 2004-2013 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2013 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "DialogBase.h"

CDialogBase::CDialogBase(void)
{
    this->hWndDialog=NULL;
    this->IconResourceId=NULL;
    this->hInstance=NULL;
    this->bDragAcceptFileRequired=FALSE;
    this->bDragAcceptFileStatusEnabled=FALSE;
    this->hWaitCursor=NULL;
    this->bWaitCursor=FALSE;
    this->bInitDone=FALSE;
    this->bInsideDialogBoxFunction=FALSE;
}

BOOL CDialogBase::EnableDragAndDrop(BOOL bEnable)
{
    if (!this->hWndDialog)
    {
        this->bDragAcceptFileRequired = TRUE;
        return TRUE;
    }
    
    HMODULE hModule = ::GetModuleHandle(CDialogBase_SHELL_DLL);
    if (!hModule)
    {
        hModule =::LoadLibrary(CDialogBase_SHELL_DLL);
        if (!hModule)
            return FALSE;
    }

    typedef void (__stdcall *pfDragAcceptFiles)(HWND hWnd, BOOL fAccept);
    pfDragAcceptFiles pDragAcceptFiles;
    pDragAcceptFiles = (pfDragAcceptFiles) ::GetProcAddress(hModule,"DragAcceptFiles");
    if (!pDragAcceptFiles)
        return FALSE;
    pDragAcceptFiles(this->hWndDialog,bEnable); // DragAcceptFiles has no return value :(
    this->bDragAcceptFileStatusEnabled = bEnable;
    return TRUE;
}

CDialogBase::~CDialogBase(void)
{
    // if we are still in dialog box message loop, that means dialog is not closed
    // --> close it before destroying object to avoid invalid memory access and software crash
    if (this->bInsideDialogBoxFunction)
    {
        this->Close();
        // wait end of Show method (aka DialogBoxParam function)
        while (this->bInsideDialogBoxFunction)
            ::Sleep(500);
    }
}

//-----------------------------------------------------------------------------
// Name: GetComboboxEditHandle
// Object: get edit control associated to combobox
// Parameters :
//     in  : int nIDDlgItem : combobox id
//     out : 
//     return : edit handle
//-----------------------------------------------------------------------------
HWND CDialogBase::GetComboboxEditHandle(IN int nIDDlgItem)
{
    HWND hCombo= this->GetDlgItem(nIDDlgItem);
    COMBOBOXINFO info = {0};
    info.cbSize = sizeof(COMBOBOXINFO);
    ::GetComboBoxInfo(hCombo, &info);
    return info.hwndItem;
}

//-----------------------------------------------------------------------------
// Name: IsComboboxHandle
// Object: check if handle belongs to combobox (test combo, list and edit)
// Parameters :
//     in  : int nIDDlgItem : combobox id
//           HWND hItemToTest : handle to test
//     out : 
//     return : TRUE if handle belongs to combo
//-----------------------------------------------------------------------------
BOOL CDialogBase::IsComboboxHandle(IN int nIDDlgItem,IN HWND hItemToTest)
{
    HWND hCombo= this->GetDlgItem(nIDDlgItem);

    if (hCombo == hItemToTest)
        return TRUE;

    COMBOBOXINFO info = {0};
    info.cbSize = sizeof(COMBOBOXINFO);
    ::GetComboBoxInfo(hCombo, &info);
    if (info.hwndItem == hItemToTest)
        return TRUE;
    if (info.hwndList == hItemToTest)
        return TRUE;
    return FALSE;
}

BOOL CDialogBase::EnableRedraw(IN BOOL bEnable)
{
    return this->EnableRedrawFromHwnd(this->hWndDialog,bEnable);
}

BOOL CDialogBase::EnableRedraw(IN int nIDDlgItem,IN BOOL bEnable)
{
    HWND hWnd= this->GetDlgItem(nIDDlgItem);
    return this->EnableRedrawFromHwnd(hWnd,bEnable);
}

BOOL CDialogBase::EnableRedrawFromHwnd(IN HWND hWnd,IN BOOL bEnable)
{
    LRESULT RetValue;
    if (bEnable)
    {
        RetValue = ::SendMessage(hWnd, WM_SETREDRAW, TRUE, 0);
        // MSDN :
        // Note  RedrawWindow with the specified flags is used instead of InvalidateRect because the former is necessary for some controls 
        // that have nonclient area on their own, or have window styles that cause them to be given a nonclient area 
        // (such as WS_THICKFRAME, WS_BORDER, or WS_EX_CLIENTEDGE). If the control does not have a nonclient area, 
        // RedrawWindow with these flags will do only as much invalidation as InvalidateRect would.
        ::RedrawWindow(hWnd, NULL, NULL, RDW_ERASE | RDW_FRAME | RDW_INVALIDATE | RDW_ALLCHILDREN);
    }
    else
    {
        // MSDN : 
        // If the application sends the WM_SETREDRAW message to a hidden window, the window becomes visible 
        // (that is, the operating system adds the WS_VISIBLE style to the window)
        RetValue = ::SendMessage(hWnd, WM_SETREDRAW, FALSE, 0);
    }
    // MSDN : An application returns zero if it processes this message
    return (RetValue == 0);
}

BOOL CDialogBase::Redraw()
{
    return this->RedrawFromHwnd(this->hWndDialog);
}

BOOL CDialogBase::Redraw(IN int nIDDlgItem)
{
    return this->RedrawFromHwnd(this->GetDlgItem(nIDDlgItem));
}

//-----------------------------------------------------------------------------
// Name: RedrawFromHwnd
// Object: redraw an item
// Parameters :
//     in  : HWND hItem : handle of item to be redraw
//     out : 
//     return : 
//-----------------------------------------------------------------------------
BOOL CDialogBase::RedrawFromHwnd(HWND hItem)
{
    return ::RedrawWindow( hItem,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME | RDW_ALLCHILDREN);
}

//-----------------------------------------------------------------------------
// Name: ScreenToClient
// Object: translate lpRect from Screen coordinates to Client Coordinates
// Parameters :
//     in out : LPRECT lpRect : in  : Rect in screen coordinates
//                              out : Rect in dialog coordinates
//     return : 
//-----------------------------------------------------------------------------
BOOL CDialogBase::ScreenToClient(IN OUT LPRECT pRect)
{
    /*
    POINT p;
    p.x=lpRect->left;
    p.y=lpRect->top;
    ScreenToClient(hDlg,&p);
    lpRect->left=p.x;
    lpRect->top=p.y;
    p.x=lpRect->right;
    p.y=lpRect->bottom;
    ScreenToClient(hDlg,&p);
    lpRect->right=p.x;
    lpRect->bottom=p.y;
    */
    if (!::ScreenToClient (this->hWndDialog, (LPPOINT)pRect))
        return FALSE;
    if(!::ScreenToClient (this->hWndDialog, ((LPPOINT)pRect)+1))
        return FALSE;
    return TRUE;
}


//-----------------------------------------------------------------------------
// Name: GetClientWindowRect
// Object: get item rect in Client Coordinates
// Parameters :
//     in  : int nIDDlgItem : item id
//     out : LPRECT lpRect : Rect in dialog coordinates
//     return : 
//-----------------------------------------------------------------------------

BOOL CDialogBase::GetClientWindowRectFromHwnd(IN HWND hItem,OUT LPRECT lpRect)
{
    if (!::GetWindowRect(hItem,lpRect))
        return FALSE;
    if(!this->ScreenToClient(lpRect))
        return FALSE;
    return TRUE;
}

BOOL CDialogBase::GetClientWindowRect(IN int nIDDlgItem,OUT LPRECT lpRect)
{
    return this->GetClientWindowRectFromHwnd(this->GetDlgItem(nIDDlgItem),lpRect);
}

//-----------------------------------------------------------------------------
// Name: ParseGroupForAction
// Object: call specified callback for each item of the group
// Parameters :
//     in  : HWND hWndGroup : group window handle
//           CDialogBase::pfCallBackGroupAction CallBackFunction : callback called for each item
//           PVOID UserParam : parameter provided to callback
//     out : 
//     return : AND between all result of callback calls
//-----------------------------------------------------------------------------
BOOL CDialogBase::ParseGroupForAction(HWND hWndGroup,pfCallBackGroupAction CallBackFunction,PVOID UserParam)
{
    BOOL bRet=TRUE;
    BOOL bFunctionRet=TRUE;
    if (!hWndGroup)
        return FALSE;
    HWND hWndCtrl;
    hWndCtrl=hWndGroup;
    // for each item of the group
    for(;;)
    {
        // call callback with item handle and UserParam
        bFunctionRet = CallBackFunction(hWndCtrl,UserParam);
        bRet=bRet && bFunctionRet;

        // get next window handle
        hWndCtrl = GetWindow( hWndCtrl, GW_HWNDNEXT );
        // if no next window or next window has WS_GROUP style (window outside of the current group)
        if ( (hWndCtrl==NULL) || (GetWindowLong(hWndCtrl,GWL_STYLE)&WS_GROUP))
            break;

    } 
    return bRet;
}

//-----------------------------------------------------------------------------
// Name: CallBackRedrawGroup
// Object: called to redraw item of a group. Call CDialogBase::RedrawFromHwnd
// Parameters :
//     in  : HWND Item : window handle
//     out : 
//     return : TRUE
//-----------------------------------------------------------------------------
BOOL CDialogBase::CallBackRedrawGroup(IN HWND Item,IN PVOID UserParam)
{
    CDialogBase* pDialogBase = (CDialogBase*)UserParam;
    pDialogBase->RedrawFromHwnd(Item);
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: RedrawGroupFromHwnd
// Object: Call CDialogBase::RedrawFromHwnd for each item of a group
// Parameters :
//     in  : HWND hWndGroup : group window handle
//     out : 
//     return : TRUE
//-----------------------------------------------------------------------------
BOOL CDialogBase::RedrawGroup(IN int nIDDlgItem)
{
    return this->RedrawGroupFromHwnd(this->GetDlgItem(nIDDlgItem));
}

BOOL CDialogBase::RedrawGroupFromHwnd(IN HWND hWndGroup)
{
    if(!hWndGroup)
        return FALSE;
    return this->ParseGroupForAction(hWndGroup,CDialogBase::CallBackRedrawGroup,this);
}

//-----------------------------------------------------------------------------
// Name: CallBackShowGroup
// Object: called to show item of a group. Call ShowWindow API
// Parameters :
//     in  : HWND Item : window handle
//           PVOID UserParam : int nCmdShow of ShowWindow API
//     out : 
//     return : TRUE
//-----------------------------------------------------------------------------
BOOL CDialogBase::CallBackShowGroup(HWND Item,PVOID UserParam)
{
    ::ShowWindow(Item,(int)UserParam);
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: ShowGroup
// Object: Call ShowWindow API for each item of a group
// Parameters :
//     in  : HWND hWndGroup : group window handle
//           int CmdShow : int nCmdShow parameter of ShowWindow API
//     out : 
//     return : TRUE
//-----------------------------------------------------------------------------
BOOL CDialogBase::ShowGroup(IN int nIDDlgItem,IN int CmdShow)
{
    HWND hWndGroup = this->GetDlgItem(nIDDlgItem);
    return this->ShowGroupFromHandle(hWndGroup,CmdShow);
}

BOOL CDialogBase::ShowGroupFromHandle(IN HWND hWndGroup,IN int CmdShow)
{
    if (!hWndGroup)
        return FALSE;
    return CDialogBase::ParseGroupForAction(hWndGroup,CDialogBase::CallBackShowGroup,(PVOID)CmdShow);
}

//-----------------------------------------------------------------------------
// Name: CallBackShowGroup
// Object: called to enable item of a group. Call ShowWindow API
// Parameters :
//     in  : HWND Item : window handle
//           PVOID UserParam : BOOL bEnable parameter of EnableWindow API
//     out : 
//     return : TRUE
//-----------------------------------------------------------------------------
BOOL CDialogBase::CallBackEnableGroup(HWND Item,PVOID UserParam)
{
    ::EnableWindow(Item,(BOOL)UserParam);
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: EnableGroup
// Object: Call EnableWindow API for each item of a group
// Parameters :
//     in  : HWND hWndGroup : group window handle
//           BOOL bEnable : BOOL bEnable parameter of EnableWindow API
//     out : 
//     return : TRUE
//-----------------------------------------------------------------------------
BOOL CDialogBase::EnableGroup(IN int nIDDlgItem,IN BOOL bEnable)
{
    HWND hWndGroup = this->GetDlgItem(nIDDlgItem);
    return this->EnableGroupFromHandle(hWndGroup,bEnable);
}
BOOL CDialogBase::EnableGroupFromHandle(IN HWND hWndGroup,IN BOOL bEnable)
{
    if (!hWndGroup)
        return FALSE;
    return CDialogBase::ParseGroupForAction(hWndGroup,CDialogBase::CallBackEnableGroup,(PVOID)bEnable);
}

//-----------------------------------------------------------------------------
// Name: CallBackMoveGroup
// Object: called to move items of a group. Call SetWindowPos API
// Parameters :
//     in  : HWND Item : window handle
//           PVOID UserParam : pointer to a MOVE_GROUP_STRUCT containing 
//                              informations to set window position
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CDialogBase::CallBackMoveGroup(HWND Item,PVOID UserParam)
{
    CDialogBase::MOVE_GROUP_STRUCT* pMoveStruct;
    RECT Rect;
    // get struct pointer
    pMoveStruct=(MOVE_GROUP_STRUCT*)UserParam;

    // get old item position
    if(!pMoveStruct->pDialogBase->GetClientWindowRectFromHwnd(Item,&Rect))
        return FALSE;
    // set new position applying displacement
    return ::SetWindowPos(Item,NULL,
                          Rect.left+pMoveStruct->Point.x,Rect.top+pMoveStruct->Point.y,
                          0,0,
                          SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
}

//-----------------------------------------------------------------------------
// Name: CallBackMoveGroup
// Object: called to move items of a group. Call SetWindowPos API
// Parameters :
//     in  : HWND hClient : handle client window from which point refer (generally handle to dialog)
//           HWND hWndGroup : handle to group window
//           POINT* pPoint : new group position in client coordinates
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CDialogBase::MoveGroupTo(IN int nIDDlgItem,IN POINT* pPoint)
{
    HWND hWndGroup = this->GetDlgItem(nIDDlgItem);
    return this->MoveGroupToFromHandle(hWndGroup,pPoint);
}
BOOL CDialogBase::MoveGroupToFromHandle(IN HWND hWndGroup,IN POINT* pPoint)
{
    RECT Rect;
    MOVE_GROUP_STRUCT MoveStruct;
    BOOL bRet;


    if (!hWndGroup)
        return FALSE;

    // get window group Rect
    this->GetClientWindowRectFromHwnd(hWndGroup,&Rect);

    // fill MOVE_GROUP_STRUCT
    MoveStruct.pDialogBase=this;
    // compute displacement values
    MoveStruct.Point.x=pPoint->x-Rect.left;
    MoveStruct.Point.y=pPoint->y-Rect.top;

    // apply displacement for all items of the group
    bRet=CDialogBase::ParseGroupForAction(hWndGroup,CDialogBase::CallBackMoveGroup,&MoveStruct);

    // redraw group
    this->RedrawFromHwnd(hWndGroup);

    return bRet;
}

HICON CDialogBase::SetStaticIcon(int StaticControlId,int IconId,int Width,int Height)
{
    HICON hIcon = this->LoadIcon(IconId,Width,Height);
    if (!hIcon)
        return hIcon;
    this->SetStaticIcon(StaticControlId,hIcon);
    return hIcon;
}

HICON CDialogBase::SetButtonIcon(int ButtonControlId,int IconId,int Width,int Height)
{
    HICON hIcon = this->LoadIcon(IconId,Width,Height);
    if (!hIcon)
        return hIcon;
    this->SetButtonIcon(ButtonControlId,hIcon);
    return hIcon;
}



BOOL CDialogBase::WaitWindowThreadEnd(HWND hWndDialog,DWORD TimeOutInMs)
{
    // Notice : assume to take only static var, so dialog object can be destroyed
    BOOL Ret;
    // get thread owing dialog proc message loop
    DWORD WindowThreadId = ::GetWindowThreadProcessId(hWndDialog,NULL);

    // avoid to wait for our self
    if (::GetCurrentThreadId() == WindowThreadId)
        return FALSE;

    // open thread for synchronization
    HANDLE hThread = ::OpenThread(SYNCHRONIZE,FALSE,WindowThreadId);
    if (!hThread)
        return FALSE;

    // wait until thread end or time out
    Ret = (::WaitForSingleObject(hThread,TimeOutInMs)== WAIT_OBJECT_0);

    ::CloseHandle(hThread);
    return Ret;
}

//-----------------------------------------------------------------------------
// Name: ReportMessage
// Object: display messagebox
// Parameters :
//     in  :
//           TCHAR* pszMsg : message to display
//           ReportMessageTypes MsgType : type of message (information, error , warning)
//     return : MessageBox return value
//-----------------------------------------------------------------------------
int CDialogBase::ReportMessage(TCHAR* pszMsg, ReportMessageTypes MsgType)
{
    TCHAR* pszCaption;
    UINT MessageBoxFlag = 0;
    switch(MsgType)
    {
    default:
    case ReportMessageType_ERROR:
        pszCaption = _T("Error");
        MessageBoxFlag = MB_ICONERROR;
        break;
    case ReportMessageType_WARNING:
        pszCaption = _T("Warning");
        MessageBoxFlag = MB_ICONWARNING;
        break;
    case ReportMessageType_INFO:
        pszCaption = _T("Information");
        MessageBoxFlag = MB_ICONINFORMATION;
        break;
    }

    return ::MessageBox(this->hWndDialog,pszMsg,pszCaption,MB_OK|MessageBoxFlag);
        
}

//-----------------------------------------------------------------------------
// Name: ReportError
// Object: display error message
// Parameters :
//     in  : 
//           TCHAR* pszMsg : error message to display
//     return : MessageBox return value
//-----------------------------------------------------------------------------
int CDialogBase::ReportError(TCHAR* pszMsg)
{
    return this->ReportMessage(pszMsg,ReportMessageType_ERROR);
}

//-----------------------------------------------------------------------------
// Name: Show
// Object: show the dialog box
// Parameters :
//     in  : HINSTANCE hInstance : application instance
//           HWND hWndDialog : main window dialog handle
//           int ResourceId : dialog resource id 
//           int IconResourceId : dialog icon resource id
//     out :
//     return : DialogBoxParam return value
//-----------------------------------------------------------------------------
INT_PTR CDialogBase::Show(HINSTANCE hInstance,HWND hWndParentDialog,int ResourceId,int IconResourceId)
{
    this->IconResourceId = IconResourceId;
    this->hInstance = hInstance;
    this->hWndParentDialog = hWndParentDialog;
    this->bInsideDialogBoxFunction = TRUE;
    INT_PTR DialogResult = ::DialogBoxParam(hInstance,MAKEINTRESOURCE(ResourceId),hWndParentDialog,CDialogBase::DlgProc,(LPARAM)this);
    // must be last object access (see destructor code)
    this->bInsideDialogBoxFunction = FALSE;

    return DialogResult;
}

//-----------------------------------------------------------------------------
// Name: Close
// Object: close the dialog box
// Parameters :
//     in  : int Code : dialog box return value
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CDialogBase::Close(INT_PTR Code)
{
    return this->CloseInternal(Code);
}
BOOL CDialogBase::CloseInternal(INT_PTR Code)
{
    if(!this->OnCloseInternal())
        return FALSE;

    if (this->bDragAcceptFileStatusEnabled)
        this->EnableDragAndDrop(FALSE);

    ::SetWindowLongPtr(this->hWndDialog,GWLP_USERDATA,(LONG_PTR)NULL);
    return ::EndDialog(this->hWndDialog,Code);
}

void CDialogBase::SetWaitCursor(BOOL bWait)
{
    if (bWait)
    {
        if (!this->hWaitCursor)
            this->hWaitCursor=::LoadCursor(NULL,IDC_WAIT);
    }
    this->bWaitCursor = bWait;

    // force cursor update
    POINT pt;
    GetCursorPos(&pt);
    SetCursorPos(pt.x,pt.y);
}

void CDialogBase::SetDlgIcon(HINSTANCE hInstance,int IconResourceId)
{
    ::SendMessage(this->hWndDialog, WM_SETICON, ICON_BIG,
                    (LPARAM)::LoadImage(hInstance, MAKEINTRESOURCE(IconResourceId),
                                        IMAGE_ICON,::GetSystemMetrics(SM_CXICON),::GetSystemMetrics(SM_CYICON),
                                        LR_DEFAULTCOLOR|LR_SHARED)
                    );
    ::SendMessage(this->hWndDialog, WM_SETICON, ICON_SMALL,
                    (LPARAM)::LoadImage(hInstance, MAKEINTRESOURCE(IconResourceId),
                                        IMAGE_ICON,::GetSystemMetrics(SM_CXSMICON),::GetSystemMetrics(SM_CYSMICON),
                                        LR_DEFAULTCOLOR|LR_SHARED)
                    );
}

DWORD CDialogBase::GetHeight()
{
    RECT Rect;
    if (::GetWindowRect(this->hWndDialog,&Rect))
    {
        return Rect.bottom-Rect.top;
    }
    //else
    return 0;
}
DWORD CDialogBase::GetWidth()
{
    RECT Rect;
    if (::GetWindowRect(this->hWndDialog,&Rect))
    {
        return Rect.right-Rect.left;
    }
    //else
    return 0;
}
DWORD CDialogBase::GetInternalHeight()
{
    RECT Rect;
    if (this->GetInternalRect(&Rect))
    {
        return Rect.bottom-Rect.top;
    }
    //else
    return 0;
}
DWORD CDialogBase::GetInternalWidth()
{
    RECT Rect;
    if (this->GetInternalRect(&Rect))
    {
        return Rect.right-Rect.left;
    }
    //else
    return 0;
}

//-----------------------------------------------------------------------------
// Name: GetAssociatedObject
// Object: Get object associated to window handle
// Parameters :
//     in  : HWND hWndDialog : handle of the window
//     out :
//     return : associated object if found, NULL if not found
//-----------------------------------------------------------------------------
CDialogBase* CDialogBase::GetAssociatedObject(HWND hWndDialog)
{
    return (CDialogBase*)GetWindowLongPtr(hWndDialog,GWLP_USERDATA);
}
//-----------------------------------------------------------------------------
// Name: DlgProc
// Object: dialog callback of the dump dialog
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
INT_PTR CALLBACK CDialogBase::DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch (uMsg)
    {
    case WM_INITDIALOG:
        {
            CDialogBase* pDialogBase=(CDialogBase*)lParam;
            pDialogBase->hWndDialog=hWnd;

            ::SetWindowLongPtr(hWnd,GWLP_USERDATA,(LONG_PTR)pDialogBase);

            if (pDialogBase->bDragAcceptFileRequired)
                pDialogBase->EnableDragAndDrop(TRUE);

            // load dlg icons
            pDialogBase->SetDlgIcon(pDialogBase->hInstance,pDialogBase->IconResourceId);

            pDialogBase->OnInitInternal();
            pDialogBase->bInitDone=TRUE;
        }
        break;
    case WM_CLOSE:
        {
            CDialogBase* pDialogBase=CDialogBase::GetAssociatedObject(hWnd);
            if (!pDialogBase)
                break;
            // 1) specific case as EDIT control in multi lines configuration send WM_CLOSE directly when edit has focus and VK_ESCAPE is pressed (known windows bug for backward compatibility)
            // 
            //LRESULT CDialogBaseChild::OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam)
            //{
            //    // specific case as EDIT control in multi lines configuration send WM_CLOSE directly when edit has focus and VK_ESCAPE is pressed (known windows bug for backward compatibility)
            //    switch (uMsg)
            //    {
            //    case WM_CLOSE:
            //        return TRUE;
            //    case WM_SYSCOMMAND:
            //        {
            //            if ( wParam == SC_CLOSE) 
            //            {
            //                this->Close();
            //            }
            //        }
            //    }
            //    return FALSE;
            //}

            // 2) take care of buttons with IDCANCEL because they send WM_CLOSE too
            if (pDialogBase->OnMessage(uMsg,wParam,lParam)) // if WM_CLOSE returns TRUE, we should not treat it
                return TRUE;
            pDialogBase->Close();
        }
        break;

    case WM_COMMAND:
        {
            CDialogBase* pDialogBase=CDialogBase::GetAssociatedObject(hWnd);
            if (!pDialogBase)
                break;
            pDialogBase->OnCommand(wParam,lParam);
        }
        break;

    case WM_NOTIFY:
        {
            CDialogBase* pDialogBase=CDialogBase::GetAssociatedObject(hWnd);
            if (!pDialogBase)
                break;
            pDialogBase->OnNotify(wParam,lParam);
        }
        break;
    case WM_DROPFILES:
        {
            CDialogBase* pDialogBase=CDialogBase::GetAssociatedObject(hWnd);
            if (!pDialogBase)
                break;
            pDialogBase->OnDropFiles(wParam,lParam);
        }
        break;
    case WM_LBUTTONDOWN:
        {
            CDialogBase* pDialogBase=CDialogBase::GetAssociatedObject(hWnd);
            if (!pDialogBase)
                break;
            pDialogBase->OnMouseDown(wParam,lParam);
        }
        break;

    case WM_LBUTTONUP:
    case WM_KILLFOCUS:
        {
            CDialogBase* pDialogBase=CDialogBase::GetAssociatedObject(hWnd);
            if (!pDialogBase)
                break;
            pDialogBase->OnMouseUp(wParam,lParam);
        }
        break;


    case WM_MOUSEMOVE:
        {
            CDialogBase* pDialogBase=CDialogBase::GetAssociatedObject(hWnd);
            if (!pDialogBase)
                break;
            pDialogBase->OnMouseMove(wParam,lParam);
        }
        break;
    case WM_SIZING:
        {
            CDialogBase* pDialogBase=CDialogBase::GetAssociatedObject(hWnd);
            if (!pDialogBase)
                break;
            pDialogBase->OnSizing(wParam,lParam);
        }
        break;
    case WM_SIZE:
        {
            CDialogBase* pDialogBase=CDialogBase::GetAssociatedObject(hWnd);
            if (!pDialogBase)
                break;
            pDialogBase->OnSize(wParam,lParam);
        }
        break;
    case WM_SETCURSOR:
        {
            CDialogBase* pDialogBase=CDialogBase::GetAssociatedObject(hWnd);
            if (!pDialogBase)
                break;

            if (pDialogBase->bWaitCursor)
            {
                ::SetCursor(pDialogBase->hWaitCursor);
                //must use SetWindowLongPtr to return value from dialog proc
                ::SetWindowLongPtr(hWnd, DWLP_MSGRESULT, TRUE);
                return TRUE;
            }
            else
            {
                ::SetWindowLongPtr(hWnd, DWLP_MSGRESULT, FALSE);
                return FALSE;
            }
            break;
        }
    default:
        {
            CDialogBase* pDialogBase=CDialogBase::GetAssociatedObject(hWnd);
            if (!pDialogBase)
                return FALSE;// we must return FALSE to tell that message is not handled
            return pDialogBase->OnMessage(uMsg,wParam,lParam);
        }
        // return FALSE;
    }
    return TRUE;
}
