/*
Copyright (C) 2004-2013 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2013 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#pragma once

#ifndef _WIN32_WINNT
    #define _WIN32_WINNT 0x0501
#endif

#ifndef DeleteAndCleanIfNotNull
    // usage : DeleteAndCleanIfNotNull(this->pObject)
    #define DeleteAndCleanIfNotNull(x) if (x){delete x; x=NULL;}
#endif

#include <windows.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)

#define CDialogBase_SHELL_DLL _T("shell32.dll")

class CDialogBase
{
protected:
    HWND hWndDialog;
    HWND hWndParentDialog;
    HINSTANCE hInstance;
    BOOL bInsideDialogBoxFunction;
    int IconResourceId;
    static INT_PTR CALLBACK DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    HCURSOR hWaitCursor;
    BOOL bWaitCursor;
    BOOL bDragAcceptFileRequired; // if we have to enable drag accept file
    BOOL bDragAcceptFileStatusEnabled;// if drag accept file is already enabled
    BOOL bInitDone;
    static BOOL WaitWindowThreadEnd(HWND hWndDialog,DWORD TimeOutInMs);

    typedef struct tagMoveGroupStruct
    {
        POINT  Point;
        CDialogBase* pDialogBase;
    }MOVE_GROUP_STRUCT,*PMOVE_GROUP_STRUCT;

    static BOOL CallBackRedrawGroup(HWND Item,PVOID UserParam);
    static BOOL CallBackShowGroup(HWND Item,PVOID UserParam);
    static BOOL CallBackEnableGroup(HWND Item,PVOID UserParam);
    static BOOL CallBackMoveGroup(HWND Item,PVOID UserParam);
    typedef BOOL (*pfCallBackGroupAction)(HWND Item,PVOID UserParam);

    BOOL ParseGroupForAction(HWND hWndGroup,pfCallBackGroupAction CallBackFunction,PVOID UserParam);
    BOOL ScreenToClient(IN OUT LPRECT pRect);
    virtual void OnInitInternal(){this->OnInit();}
    virtual BOOL OnCloseInternal(){return this->OnClose();}
    BOOL CloseInternal(INT_PTR Code);
public:
    CDialogBase(void);
    virtual ~CDialogBase(void);
        static CDialogBase* GetAssociatedObject(HWND hWndDialog);
    FORCEINLINE BOOL IsInsideDialogBoxFunction(){return this->bInsideDialogBoxFunction;}
    FORCEINLINE BOOL IsInitialized(){return this->bInitDone;}// return TRUE after OnInit() completion
    virtual void OnInit(){}
    virtual BOOL OnClose(){return TRUE;}//return FALSE to cancel closing
    virtual void OnCommand(WPARAM wParam,LPARAM lParam)
    {
        UNREFERENCED_PARAMETER(lParam);
        switch(LOWORD(wParam))
        {
            case IDOK:
                this->Close(IDOK);
                break;
            case IDCANCEL:
                this->Close(IDCANCEL);
                break;
        }
    }
    virtual void OnNotify(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);}
    virtual void OnDropFiles(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);}
    virtual void OnMouseDown(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);}
    virtual void OnMouseUp(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);}
    virtual void OnMouseMove(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);}
    virtual void OnSize(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);}
    virtual void OnSizing(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);}
    virtual LRESULT OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(uMsg);UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);return FALSE;}

    virtual INT_PTR Show(HINSTANCE hInstance,HWND hWndParentDialog,int ResourceId,int IconResourceId);
    // virtual BOOL Close(INT_PTR Code = 0);
    virtual BOOL Close(){return this->Close(0);}
    virtual BOOL Close(INT_PTR Code);

    FORCEINLINE HWND GetControlHandle(){return this->hWndDialog;}
    FORCEINLINE BOOL GetInternalRect(LPRECT lpRect){return ::GetClientRect(this->hWndDialog,lpRect);}
    FORCEINLINE BOOL GetWindowRect(LPRECT lpRect){return ::GetWindowRect(this->hWndDialog,lpRect);}
    DWORD GetHeight();
    DWORD GetWidth();
    DWORD GetInternalHeight();
    DWORD GetInternalWidth();
    FORCEINLINE HINSTANCE GetInstance(){return this->hInstance;}
    FORCEINLINE HWND GetDlgItem(IN int nIDDlgItem){return ::GetDlgItem(this->hWndDialog,nIDDlgItem);}
    FORCEINLINE LRESULT SendDlgMessage(IN int nIDDlgItem,IN UINT message,IN WPARAM wParam,IN LPARAM lParam){return ::SendMessage(::GetDlgItem(this->hWndDialog,nIDDlgItem),message,wParam,lParam);}
    FORCEINLINE LRESULT PostDlgMessage(IN int nIDDlgItem,IN UINT message,IN WPARAM wParam,IN LPARAM lParam){return ::PostMessage(::GetDlgItem(this->hWndDialog,nIDDlgItem),message,wParam,lParam);}

    FORCEINLINE BOOL SetDlgItemInt(IN int nIDDlgItem,IN UINT uValue,IN BOOL bSigned){return ::SetDlgItemInt(this->hWndDialog, nIDDlgItem, uValue, bSigned);}
    FORCEINLINE UINT GetDlgItemInt(IN int nIDDlgItem,OUT BOOL* lpTranslated,IN BOOL bSigned){return ::GetDlgItemInt(this->hWndDialog, nIDDlgItem, lpTranslated, bSigned);}
    FORCEINLINE BOOL SetDlgItemText(IN int nIDDlgItem,LPCTSTR lpszString){return ::SetWindowText(::GetDlgItem(this->hWndDialog,nIDDlgItem), lpszString);}
    // nMaxCount : Specifies the maximum number of TCHARs to be copied, including the terminating null character
    FORCEINLINE int GetDlgItemText(IN int nIDDlgItem,OUT LPTSTR lpString,IN int nMaxCount){*lpString=0;return ::GetWindowText(::GetDlgItem(this->hWndDialog,nIDDlgItem), lpString,nMaxCount);}
    // The return value is the length of the text in TCHARs, not including the terminating null character
    FORCEINLINE int GetDlgItemTextLength(IN int nIDDlgItem){return ::GetWindowTextLength(::GetDlgItem(this->hWndDialog,nIDDlgItem));}

    FORCEINLINE BOOL IsDlgButtonChecked(IN int nIDDlgItem){return (::SendMessage(::GetDlgItem(this->hWndDialog,nIDDlgItem),BM_GETCHECK,0,0)!=BST_UNCHECKED);}
    FORCEINLINE LRESULT SetDlgButtonCheckState(IN int nIDDlgItem,IN BOOL bChecked){return ::SendMessage(::GetDlgItem(this->hWndDialog,nIDDlgItem),BM_SETCHECK,bChecked ? BST_CHECKED : BST_UNCHECKED,0);}
    FORCEINLINE BOOL EnableDlgItem(IN int nIDDlgItem,IN BOOL bEnable){return ::EnableWindow(::GetDlgItem(this->hWndDialog,nIDDlgItem),bEnable);}

    // do not call Destroy icon for returned icon handle
    FORCEINLINE HICON LoadIcon(int IconId,int Width,int Height){return (HICON)::LoadImage(this->hInstance,MAKEINTRESOURCE(IconId),IMAGE_ICON,Width,Height,LR_DEFAULTCOLOR|LR_SHARED);}
    FORCEINLINE LRESULT SetStaticIcon(int StaticControlId,HICON hIcon){return ::SendMessage(::GetDlgItem(this->hWndDialog,StaticControlId),STM_SETICON,(WPARAM)hIcon,0);}
    HICON SetStaticIcon(int StaticControlId,int IconId,int Width,int Height);
    FORCEINLINE LRESULT SetButtonIcon(int ButtonControlId,HICON hIcon){return ::SendMessage(::GetDlgItem(this->hWndDialog,ButtonControlId),BM_SETIMAGE,(WPARAM)IMAGE_ICON,(LPARAM)hIcon);}
    HICON SetButtonIcon(int ButtonControlId,int IconId,int Width,int Height);
    
    void SetDlgIcon(HINSTANCE hInstance,int IconResourceId);
    FORCEINLINE BOOL SetCaption(TCHAR* DlgCaption){return ::SetWindowText(this->hWndDialog, DlgCaption);}
    void SetWaitCursor(BOOL bWaitCursor);
    
    FORCEINLINE BOOL Hide(){return ::ShowWindow(this->hWndDialog,SW_HIDE);}
    FORCEINLINE BOOL Hide(IN int nIDDlgItem){return ::ShowWindow(this->GetDlgItem(nIDDlgItem),SW_HIDE);}
    FORCEINLINE BOOL Hide(IN HWND hWnd){return ::ShowWindow(hWnd,SW_HIDE);}

    FORCEINLINE BOOL Show(){return ::ShowWindow(this->hWndDialog,SW_SHOWNORMAL);}
    FORCEINLINE BOOL Show(IN int nIDDlgItem){return ::ShowWindow(this->GetDlgItem(nIDDlgItem),SW_SHOWNORMAL);}
    FORCEINLINE BOOL Show(IN int nIDDlgItem,IN int CmdShow){return ::ShowWindow(this->GetDlgItem(nIDDlgItem),CmdShow);}
    FORCEINLINE BOOL Show(IN HWND hWnd){return ::ShowWindow(hWnd,SW_SHOWNORMAL);}

    FORCEINLINE BOOL IsVisible(){return ::IsWindowVisible(this->hWndDialog);}

    FORCEINLINE BOOL BringToTop(){return ::BringWindowToTop(this->hWndDialog);}
    FORCEINLINE BOOL SetForeground(){return ::SetForegroundWindow(this->hWndDialog);}
    FORCEINLINE BOOL SetFocus(HWND hWndControl){return ::PostMessage(this->hWndDialog, WM_NEXTDLGCTL, (WPARAM)hWndControl, TRUE);}// we must use WM_NEXTDLGCTL for a dialog proc

    // get window thread and pid from handle
    FORCEINLINE DWORD GetWindowThreadID(){return ::GetWindowThreadProcessId(this->hWndDialog,NULL);}
    FORCEINLINE DWORD GetWindowProcessID(){DWORD ProcessId = NULL; ::GetWindowThreadProcessId(this->hWndDialog,&ProcessId);return ProcessId;}
    FORCEINLINE BOOL WaitWindowThreadEnd(DWORD TimeOutInMs){CDialogBase::WaitWindowThreadEnd(this->hWndDialog,TimeOutInMs);}

    FORCEINLINE LONG_PTR SetStyle(IN LONG_PTR NewStyle){return ::SetWindowLongPtr(this->hWndDialog,GWL_STYLE,NewStyle);}
    FORCEINLINE LONG_PTR SetStyle(IN int nIDDlgItem,IN LONG_PTR NewStyle){return ::SetWindowLongPtr(this->GetDlgItem(nIDDlgItem),GWL_STYLE,NewStyle);}
    FORCEINLINE LONG_PTR SetStyleFromHwnd(IN HWND hItem,IN LONG_PTR NewStyle){return ::SetWindowLongPtr(hItem,GWL_STYLE,NewStyle);}

    FORCEINLINE LONG_PTR GetStyle(){return ::GetWindowLongPtr(this->hWndDialog,GWL_STYLE);}
    FORCEINLINE LONG_PTR GetStyle(IN int nIDDlgItem){return ::GetWindowLongPtr(this->GetDlgItem(nIDDlgItem),GWL_STYLE);}
    FORCEINLINE LONG_PTR GetStyleFromHwnd(IN HWND hItem){return ::GetWindowLongPtr(hItem,GWL_STYLE);}

    FORCEINLINE LONG_PTR SetStyleEx(IN LONG_PTR NewStyle){return ::SetWindowLongPtr(this->hWndDialog,GWL_EXSTYLE,NewStyle);}
    FORCEINLINE LONG_PTR SetStyleEx(IN int nIDDlgItem,IN LONG_PTR NewStyle){return ::SetWindowLongPtr(this->GetDlgItem(nIDDlgItem),GWL_EXSTYLE,NewStyle);}
    FORCEINLINE LONG_PTR SetStyleExFromHwnd(IN HWND hItem,IN LONG_PTR NewStyle){return ::SetWindowLongPtr(hItem,GWL_EXSTYLE,NewStyle);}

    FORCEINLINE LONG_PTR GetStyleEx(){return ::GetWindowLongPtr(this->hWndDialog,GWL_EXSTYLE);}
    FORCEINLINE LONG_PTR GetStyleEx(IN int nIDDlgItem){return ::GetWindowLongPtr(this->GetDlgItem(nIDDlgItem),GWL_EXSTYLE);}
    FORCEINLINE LONG_PTR GetStyleExFromHwnd(IN HWND hItem){return ::GetWindowLongPtr(hItem,GWL_EXSTYLE);}

    BOOL EnableDragAndDrop(BOOL bEnable);

    BOOL EnableRedraw(IN BOOL bEnable);
    BOOL EnableRedraw(IN int nIDDlgItem,IN BOOL bEnable);
    BOOL EnableRedrawFromHwnd(IN HWND hWnd,IN BOOL bEnable);

    BOOL Redraw();
    BOOL Redraw(IN int nIDDlgItem);
    BOOL RedrawFromHwnd(HWND hItem);
    BOOL GetClientWindowRect(IN int nIDDlgItem,OUT LPRECT lpRect);
    BOOL GetClientWindowRectFromHwnd(IN HWND hItem,OUT LPRECT lpRect);

    FORCEINLINE BOOL SetPosition(LONG x,LONG y){return ::SetWindowPos(this->hWndDialog,0,x,y,0,0,SWP_NOZORDER | SWP_NOSIZE | SWP_NOACTIVATE);}
    FORCEINLINE BOOL SetPositionFromHwnd(HWND hWnd,LONG x,LONG y){return ::SetWindowPos(hWnd,0,x,y,0,0,SWP_NOZORDER | SWP_NOSIZE | SWP_NOACTIVATE);}
    FORCEINLINE BOOL SetRect(RECT* pRect){return ::SetWindowPos(this->hWndDialog,NULL,pRect->left,pRect->top,pRect->right-pRect->left,pRect->bottom-pRect->top,SWP_NOACTIVATE | SWP_NOZORDER);}
    FORCEINLINE BOOL SetRectFromHwnd(HWND hWnd,RECT* pRect){return ::SetWindowPos(hWnd,NULL,pRect->left,pRect->top,pRect->right-pRect->left,pRect->bottom-pRect->top,SWP_NOACTIVATE | SWP_NOZORDER);}
    FORCEINLINE BOOL SetSize(LONG Width,LONG Height){return ::SetWindowPos(this->hWndDialog,NULL,0,0,Width,Height,SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);}
    FORCEINLINE BOOL SetSizeFromHwnd(HWND hWnd,LONG Width,LONG Height){return ::SetWindowPos(hWnd,NULL,0,0,Width,Height,SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);}

    ////////////////
    // Combo box
    ////////////////
    HWND GetComboboxEditHandle(IN int nIDDlgItem);
    BOOL IsComboboxHandle(IN int nIDDlgItem,IN HWND hItemToTest);

    /////////////////
    // group actions
    /////////////////
    BOOL RedrawGroup(IN int nIDDlgItem);
    BOOL RedrawGroupFromHwnd(IN HWND hWndGroup);
    BOOL ShowGroup(IN int nIDDlgItem,IN int CmdShow);
    BOOL ShowGroupFromHandle(IN HWND hWndGroup,IN int CmdShow);
    BOOL EnableGroup(IN int nIDDlgItem,IN BOOL bEnable);
    BOOL EnableGroupFromHandle(IN HWND hWndGroup,IN BOOL bEnable);
    BOOL MoveGroupTo(IN int nIDDlgItem,IN POINT* pPoint);
    BOOL MoveGroupToFromHandle(IN HWND hWndGroup,IN POINT* pPoint);


    enum ReportMessageTypes
    {
        ReportMessageType_ERROR,
        ReportMessageType_WARNING,
        ReportMessageType_INFO
    };
    int ReportMessage(TCHAR* pszMsg, ReportMessageTypes MsgType);
    int ReportError(TCHAR* pszMsg);
};
