/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class helper for NotifyIcon
//-----------------------------------------------------------------------------

#include "NotifyIcon.h"

#define ARRAY_SIZE(x)	(sizeof(x)/sizeof(x[0]))
UINT CurrentID=0; // max id used by all CNotifyIcon instances (unique for each different instance)
const UINT WM_TASKBARCREATED = ::RegisterWindowMessage(_T("TaskbarCreated"));


//-----------------------------------------------------------------------------
// Name: CNotifyIcon
// Object: Constructor
// Parameters :
//     in  : HINSTANCE hInstance : instance
//           int IdIcon : resource icon id
//           TCHAR* szTip : notify icon tooltip
//     out :
//     return : 
//-----------------------------------------------------------------------------
#ifdef _WIN64
CNotifyIcon::CNotifyIcon(HINSTANCE hInstance,int IdIcon,TCHAR* szTip,INotifyIconEvents* pEventsHandler)
{
    HICON hIcon=(HICON)::LoadImage(hInstance, MAKEINTRESOURCE(IdIcon),IMAGE_ICON,0,0,LR_SHARED);
    this->CommonConstructor(hInstance,hIcon,szTip,pEventsHandler);
}
#endif

//-----------------------------------------------------------------------------
// Name: CNotifyIcon
// Object: Constructor
// Parameters :
//     in  : HINSTANCE hInstance : instance
//           HICON hIcon : icon handle
//           TCHAR* szTip : notify icon tooltip
//     out :
//     return : 
//-----------------------------------------------------------------------------
CNotifyIcon::CNotifyIcon(HINSTANCE hInstance,HICON hIcon,TCHAR* szTip,INotifyIconEvents* pEventsHandler)
{
    this->CommonConstructor(hInstance,hIcon,szTip,pEventsHandler);
}

//-----------------------------------------------------------------------------
// Name: CommonConstructor
// Object: common Constructor
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CNotifyIcon::CommonConstructor(HINSTANCE hInstance,HICON hIcon,TCHAR* szTip,INotifyIconEvents* pEventsHandler)
{
    this->hInstance=hInstance;
    this->pEventsHandler = pEventsHandler;
    this->hWindow=NULL;

    memset(&this->NotifyIconData,0,sizeof(NOTIFYICONDATA));
    this->NotifyIconData.cbSize=sizeof(NOTIFYICONDATA);
    this->NotifyIconData.hIcon=hIcon;
    this->NotifyIconData.uCallbackMessage=CNotifyIcon_WM_MESSAGE_ID;
    this->NotifyIconData.uID=CurrentID++;
    this->NotifyIconData.uVersion=NOTIFYICON_VERSION;
    _tcsncpy(this->NotifyIconData.szTip,szTip,ARRAY_SIZE(this->NotifyIconData.szTip));

    this->Visible=FALSE;
    this->LastEventIsDoubleClick=FALSE;

    this->CreateControl();
}

CNotifyIcon::~CNotifyIcon(void)
{
    this->Hide();
    ::RemoveProp(this->hWindow,CNotifyIcon_MARKER_PROP_NAME);
    ::DestroyWindow(this->hWindow);
}

//-----------------------------------------------------------------------------
// Name: GetAssocatedNotifyIconHwnd
// Object: get the hwnd of the window associated to notify icon. (can be use by the way if you want to 
//          show a popup menu without creating any window)
// Parameters :
//     in  : 
//     out :
//     return : TRUE on Success, FALSE on Error
//-----------------------------------------------------------------------------
HWND CNotifyIcon::GetAssocatedNotifyIconHwnd()
{
    return this->hWindow;
}

//-----------------------------------------------------------------------------
// Name: CreateControl
// Object: create the control
// Parameters :
//     in  : 
//     out :
//     return : TRUE on Success, FALSE on Error
//-----------------------------------------------------------------------------
BOOL CNotifyIcon::CreateControl()
{
    // create an hidden window containing nothing
    WNDCLASSEX	wc;
    if (!::GetClassInfoEx (this->hInstance, CNotifyIcon_NOTIFYICON_CLASSEX, &wc))
    {
        ZeroMemory (&wc, sizeof(wc));
        wc.cbSize	     = sizeof (wc);
        wc.cbWndExtra    = 0;
        wc.hInstance     = this->hInstance;
        wc.hCursor	     = ::LoadCursor (NULL, IDC_ARROW);
        wc.hbrBackground = NULL;
        wc.lpfnWndProc   = CNotifyIcon::WindowProc;
        wc.lpszClassName = CNotifyIcon_NOTIFYICON_CLASSEX;
        ::RegisterClassEx (&wc);
    }
    this->hWindow = ::CreateWindowEx (
        NULL,
        CNotifyIcon_NOTIFYICON_CLASSEX, // Window class name
        NULL,                           // Window name
        NULL,                           // Window style
        // Position & size
        0,0,
        0,0,
        NULL,		        // Parent window handle
        (HMENU)NULL,		// Application defined ID must be unique for all child windows with the same parent window
        hInstance,			// Application instance
        this);				// Parameter

    if (!this->hWindow)
        return FALSE;

    this->NotifyIconData.hWnd=this->hWindow;

    // put CNotifyIcon object in window parameter
    if (!::SetProp((HWND)this->hWindow,CNotifyIcon_MARKER_PROP_NAME,(HANDLE)this))
        return FALSE;

    return TRUE;
}


//-----------------------------------------------------------------------------
// Name: AfterCallBackActions
// Object: Taskbar/NotifyIcon required action after interface operations
// Parameters :
//     in  : BOOL RestoreFocuseToNotifyIcon : TRUE to restore focus to notify icon
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CNotifyIcon::AfterCallBackActions(BOOL RestoreFocuseToNotifyIcon)
{
    if (RestoreFocuseToNotifyIcon)
    {
        // MSDN
        // NIM_SETFOCUS : Taskbar icons should use this message when they have completed their user interface operation.
        // For example, if the taskbar icon displays a shortcut menu, but the user presses ESC to cancel it,
        // use NIM_SETFOCUS to return focus to the taskbar notification area.
        this->NotifyIconData.uFlags=0;
        ::Shell_NotifyIcon(NIM_SETFOCUS,&this->NotifyIconData);
    }
}

//-----------------------------------------------------------------------------
// Name: SetIcon
// Object: change icon of NotifyIcon
// Parameters :
//     in  : HMODULE hModule : handle of module containing resource icon
//           int IdIcon : resource icon id
//     out :
//     return : TRUE on Success, FALSE on Error
//-----------------------------------------------------------------------------
BOOL CNotifyIcon::SetIcon(HMODULE hModule,int IdIcon)
{
    HICON hIcon=(HICON)::LoadImage(hModule, MAKEINTRESOURCE(IdIcon),IMAGE_ICON,0,0,LR_SHARED);
    return this->SetIcon(hIcon);
}

//-----------------------------------------------------------------------------
// Name: SetIcon
// Object: change icon of NotifyIcon
// Parameters :
//     in  : HICON hIcon : icon handle
//     out :
//     return : TRUE on Success, FALSE on Error
//-----------------------------------------------------------------------------
BOOL CNotifyIcon::SetIcon(HICON hIcon)
{
    if (!hIcon)
        return FALSE;

    // update class members
    this->NotifyIconData.hIcon=hIcon;

    // if notify icon is visible
    if (this->Visible)
    {
        // update icon
        this->NotifyIconData.uFlags=NIF_ICON;
        return ::Shell_NotifyIcon(NIM_MODIFY,&this->NotifyIconData);
    }

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SetTip
// Object: change tip of NotifyIcon
// Parameters :
//     in  : TCHAR* szTip : tip
//     out :
//     return : TRUE on Success, FALSE on Error
//-----------------------------------------------------------------------------
BOOL CNotifyIcon::SetTip(TCHAR* szTip)
{
    if (IsBadReadPtr(szTip,sizeof(TCHAR)))
        return FALSE;

    _tcsncpy(this->NotifyIconData.szTip,szTip,ARRAY_SIZE(this->NotifyIconData.szTip));

    // if notify icon is visible
    if (this->Visible)
    {
        // update icon
        this->NotifyIconData.uFlags=NIF_TIP;
        return ::Shell_NotifyIcon(NIM_MODIFY,&this->NotifyIconData);
    }

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: GetAssociatedObject
// Object: Get object associated to window handle
// Parameters :
//     in  : HWND hWndDialog : handle of the window
//     out :
//     return : associated object if found, NULL if not found
//-----------------------------------------------------------------------------
CNotifyIcon* CNotifyIcon::GetAssociatedObject(HWND hWndDialog)
{
    return (CNotifyIcon*)::GetProp(hWndDialog,CNotifyIcon_MARKER_PROP_NAME);
}



//-----------------------------------------------------------------------------
// Name: Show
// Object: show notify icon
// Parameters :
//     in  : 
//     out :
//     return : TRUE on Success, FALSE on Error
//-----------------------------------------------------------------------------
BOOL CNotifyIcon::Show()
{
    // if icon is already visible
    if (this->Visible)
        return TRUE;

    // assume a valid hIcon was provided
    if (!this->NotifyIconData.hIcon)
        return FALSE;

    // set flag for icon, WindowProc message, and tooltip 
    this->NotifyIconData.uFlags=NIF_ICON|NIF_MESSAGE|NIF_TIP;

    // display icon
    this->Visible=::Shell_NotifyIcon(NIM_ADD,&this->NotifyIconData);

    return this->Visible;
}

//-----------------------------------------------------------------------------
// Name: Hide
// Object: Hide notify icon
// Parameters :
//     in  : 
//     out :
//     return : TRUE on Success, FALSE on Error
//-----------------------------------------------------------------------------
BOOL CNotifyIcon::Hide()
{
    // if icon is already hidden
    if (!this->Visible)
        return TRUE;

    this->Visible=FALSE;

    // remove icon
    return ::Shell_NotifyIcon(NIM_DELETE,&this->NotifyIconData);
}

//-----------------------------------------------------------------------------
// Name: ShowBallon
// Object: show notify icon balloon
// Parameters :
//     in  : LPCTSTR szMsg : message to be displayed in balloon
//           LPCTSTR szTitle : balloon title
//           UINT uTimeout : timeout for balloon (or INFINITE)
//           tagBalloonIcon BalloonIcon : balloon icon type
//     out :
//     return : TRUE on Success, FALSE on Error
//-----------------------------------------------------------------------------
BOOL CNotifyIcon::ShowBallon(LPCTSTR szMsg, LPCTSTR szTitle,UINT uTimeout, tagBalloonIcon BalloonIcon)
{
    // fill NotifyIconData struct members
    _tcsncpy(this->NotifyIconData.szInfo,szMsg,ARRAY_SIZE(this->NotifyIconData.szInfo));
    _tcsncpy(this->NotifyIconData.szInfoTitle,szTitle,ARRAY_SIZE(this->NotifyIconData.szInfoTitle));
    this->NotifyIconData.uTimeout=uTimeout;
    this->NotifyIconData.dwInfoFlags=BalloonIcon;
    this->NotifyIconData.uFlags=NIF_INFO;
    
    // show balloon
    return ::Shell_NotifyIcon(NIM_MODIFY,&this->NotifyIconData);
}

//-----------------------------------------------------------------------------
// Name: WindowProc
// Object: window proc
// Parameters :
//     in  : WndProc param
//     out : 
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK CNotifyIcon::WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
    case CNotifyIcon_WM_MESSAGE_ID:
        {
            // get pointer of object associated to notify icon
            CNotifyIcon* pNotifyIcon=CNotifyIcon::GetAssociatedObject(hWnd);
            BOOL bRestoreFocuseToNotifyIcon;
            if (pNotifyIcon)
            {
                // assume message is for our control
                if (hWnd==pNotifyIcon->NotifyIconData.hWnd)
                {
                    switch(lParam)
                    {
                    case WM_CONTEXTMENU:
                    case WM_RBUTTONUP:
                        if(pNotifyIcon->pEventsHandler)
                        {
                            // get mouse position
                            POINT p;
                            ::GetCursorPos(&p);

                            // call callback
                            bRestoreFocuseToNotifyIcon=pNotifyIcon->pEventsHandler->OnNotifyIconRightClick(pNotifyIcon,&p);

                            // do required job after callback
                            pNotifyIcon->AfterCallBackActions(bRestoreFocuseToNotifyIcon);
                        }
                        break;
                    case WM_LBUTTONDBLCLK:
                        // put flag to avoid LeftClickCallBack to be called after a double click 
                        pNotifyIcon->LastEventIsDoubleClick=TRUE;

                        if(pNotifyIcon->pEventsHandler)
                        {
                            // get mouse position
                            POINT p;
                            ::GetCursorPos(&p);

                            // call callback
                            bRestoreFocuseToNotifyIcon=pNotifyIcon->pEventsHandler->OnNotifyIconDoubleClick(pNotifyIcon,&p);

                            // do required job after callback
                            pNotifyIcon->AfterCallBackActions(bRestoreFocuseToNotifyIcon);
                        }
                        
                        break;
                    case WM_LBUTTONUP:
                        // check flag to avoid LeftClickCallBack to be called after a double click 
                        if (!pNotifyIcon->LastEventIsDoubleClick)
                        {   
                            if(pNotifyIcon->pEventsHandler)
                            {
                                // get mouse position
                                POINT p;
                                ::GetCursorPos(&p);

                                // call callback
                                bRestoreFocuseToNotifyIcon=pNotifyIcon->pEventsHandler->OnNotifyIconClick(pNotifyIcon,&p);

                                // do required job after callback
                                pNotifyIcon->AfterCallBackActions(bRestoreFocuseToNotifyIcon);
                            }
                        }

                        // reset flag to avoid LeftClickCallBack to be called after a double click 
                        pNotifyIcon->LastEventIsDoubleClick=FALSE;
                        break;
                    }
                }
            }
        }
        break;
    default:
        if (uMsg==WM_TASKBARCREATED)
        {
            // get pointer of object associated to notify icon
            CNotifyIcon* pNotifyIcon=CNotifyIcon::GetAssociatedObject(hWnd);
            if (pNotifyIcon)
            {
                // if icon was displayed
                if (pNotifyIcon->Visible)
                {
                    // reset our internal flag
                    pNotifyIcon->Visible=FALSE;

                    // query icon to be redrawn
                    pNotifyIcon->Show();
                }
            }
        }

        // return the default action return
        return ::DefWindowProc (hWnd, uMsg, wParam, lParam);
    }
    return 0;
}
