/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class helper for NotifyIcon
//-----------------------------------------------------------------------------

#pragma once

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0501 // for xp os
#endif

#ifndef _WIN32_IE
#define _WIN32_IE 0x0500 // should be at least ie 5
#endif

#include <windows.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)

#include <shellapi.h>
#pragma comment(lib,"shell32")

#define CNotifyIcon_NOTIFYICON_CLASSEX  _T("CNotifyIcon_CLASSEX")
#define CNotifyIcon_MARKER_PROP_NAME    _T("CNotifyIcon_MARKER_PROP_NAME")
#define CNotifyIcon_WM_MESSAGE_ID       (WM_USER+1)

class CNotifyIcon;
class INotifyIconEvents
{
public:
    // return TRUE to restore focus to notify icon, FALSE to keep focus
    virtual BOOL OnNotifyIconClick(CNotifyIcon* pSender,LPPOINT pCursorPos)=0;
    // return TRUE to restore focus to notify icon, FALSE to keep focus
    virtual BOOL OnNotifyIconRightClick(CNotifyIcon* pSender,LPPOINT pCursorPos)=0;
    // return TRUE to restore focus to notify icon, FALSE to keep focus
    virtual BOOL OnNotifyIconDoubleClick(CNotifyIcon* pSender,LPPOINT pCursorPos)=0;
};

class CNotifyIconEventsBase:public INotifyIconEvents
{
public:
    // return TRUE to restore focus to notify icon, FALSE to keep focus
    virtual BOOL OnNotifyIconClick(CNotifyIcon* pSender,LPPOINT pCursorPos)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(pCursorPos);
        return TRUE;
    }
    // return TRUE to restore focus to notify icon, FALSE to keep focus
    virtual BOOL OnNotifyIconRightClick(CNotifyIcon* pSender,LPPOINT pCursorPos)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(pCursorPos);
        return TRUE;
    }
    // return TRUE to restore focus to notify icon, FALSE to keep focus
    virtual BOOL OnNotifyIconDoubleClick(CNotifyIcon* pSender,LPPOINT pCursorPos)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(pCursorPos);
        return TRUE;
    }
};

class CNotifyIcon
{
private:
    HINSTANCE hInstance;
    HWND hWindow;
    BOOL Visible;
    BOOL LastEventIsDoubleClick;
    INotifyIconEvents* pEventsHandler;

    NOTIFYICONDATA NotifyIconData;
    
    static CNotifyIcon* GetAssociatedObject(HWND hWndDialog);
    static LRESULT CALLBACK WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    void AfterCallBackActions(BOOL RestoreFocuseToNotifyIcon);
    BOOL CreateControl();
    void CommonConstructor(HINSTANCE hInstance,HICON hIcon,TCHAR* szTip,INotifyIconEvents* pEventsHandler);

public:

    enum tagBalloonIcon
    {
        BALLOON_ICON_ERROR=NIIF_ERROR,
        BALLOON_ICON_INFO=NIIF_INFO,
        BALLOON_ICON_NONE=NIIF_NONE,
        BALLOON_ICON_WARNING=NIIF_WARNING
    };

#ifdef _WIN64
    // on win32 compilation troubles between int and HICON
    CNotifyIcon(HINSTANCE hInstance,int IdIcon,TCHAR* szTip,INotifyIconEvents* pEventsHandler);
#endif
    CNotifyIcon(HINSTANCE hInstance,HICON hIcon,TCHAR* szTip,INotifyIconEvents* pEventsHandler);
    ~CNotifyIcon(void);
    BOOL Show();
    BOOL Hide();
    BOOL SetIcon(HMODULE hModule,int IdIcon);
    BOOL SetIcon(HICON hIcon);
    BOOL SetTip(TCHAR* szTip);
    BOOL ShowBallon(LPCTSTR szMsg, LPCTSTR szTitle,UINT uTimeout, tagBalloonIcon BalloonIcon);

    HWND GetAssocatedNotifyIconHwnd();
};
